# React SDK for Zocks.io

`@zocks-communications/client-sdk-react` is the official React SDK for [Zocks.io](https://zocks.io). With this SDK, you
can quickly integrate functionality provided by Zocks.io into your React application. The SDK is built on top of the
[Javascript Zocks.io SDK](https://www.npmjs.com/package/@zocks-communications/client-sdk-js)

The SDK is a collection of hooks and standard components. The components are highly customizable and extendable via both
CSS and render props. Hooks are provided for using the SDK in headless mode.

## Docs

Docs are available at https://docs.zocks.io

[React SDK Reference](https://docs.zocks.io/category/react)

## Installation

### Yarn

```shell
yarn add @zocks-communications/client-sdk-react
```

### npm

```shell
npm install @zocks-communications/client-sdk-react --save
```

# Usage

## Example

A simple meeting application can be created as follows:

`Meeting.tsx` (_accountId_ and _apiBaseUrl_ should be filled)

```typescript jsx
import {
    useZocksActions,
    useZocksSession,
    useLeaveOnUnload,
    JoinPage,
    Participants,
    RoomControls
} from '@zocks-communications/client-sdk-react'

export const Meeting = () => {
    const {sessionActions, localMediaActions} = useZocksActions({accountId: '{accountId}'})
    useLeaveOnUnload(sessionActions.leave)

    const {localParticipantIsConnected} = useZocksSession()

    return (
        <div className='meeting'>
            {!localParticipantIsConnected ? (
                <JoinPage createRoomAction={sessionActions.createRoom}
                          joinAction={sessionActions.join}
                />
            ) : (
                <>
                    <RoomControls mediaActions={localMediaActions} leaveAction={sessionActions.leave}/>
                    <Participants/>
                </>
            )}
        </div>
    )
}
```

Import the global library CSS and wrap your component into `ZocksProvider`

`App.tsx`

```typescript jsx
/* ... */
import '@zocks-communications/client-sdk-react/dist/zocks-client-sdk-react.css'

const App: React.FC = () => {
    return (
        <ZocksProvider>
            <Meeting/>
        </ZocksProvider>
    )
}
```

Then, simply run the application, and the meeting app should start.

## Hooks

#### useZocksActions

This hook is used for accessing session, local media and chat related actions. The actions automatically handle joining
to a session,
processing events, publishing audio, video or screen share tracks, and more.

The hook also exposes the low-level Zocks Session after connecting for more fine-grained control over the session if
necessary, for example for custom event handling.

Parameters:

|   Parameter   |            Type             | Description              |
|:-------------:|:---------------------------:|--------------------------|
| `zocksConfig` | [ZocksConfig](#zocksconfig) | The Zocks configuration. |

Returns a [ZocksActions](#ZocksActions) object. For more detail on the specific actions, see the corresponding model
objects.

|        Property        |                          Type                           | Description                    |
|:----------------------:|:-------------------------------------------------------:|--------------------------------|
|    `sessionActions`    |       [ZocksSessionActions](#zockssessionactions)       | Session related actions.       |
|  `localMediaActions`   |         [ZocksMediaActions](#zocksmediaactions)         | Local media related actions.   |
|     `chatActions`      |          [ZocksChatActions](#zockschatactions)          | Chat related actions.          |
| `transcriptionActions` | [ZocksTranscriptionActions](#zockstranscriptionactions) | Transcription related actions. |
|       `session`        |                Session &#124; undefined                 | The current Zocks session.     |

---

#### useZocksSession

Provides state data about the current session.

Returns:

|           Property            |                  Type                   | Description                                                                       |
|:-----------------------------:|:---------------------------------------:|-----------------------------------------------------------------------------------|
|          `sessionId`          |           string &#124; null            | Current session identifier.                                                       |
|          `roomState`          |                RoomState                | Room state.                                                                       |
|      `localParticipant`       |  [ParticipantModel](#participantmodel)  | Local participant.                                                                |
|     `remoteParticipants`      | [ParticipantModel](#participantmodel)[] | Remote participants.                                                              |
|        `isConnecting`         |                 boolean                 | Flag indicating if connection is in progress or not.                              |
| `localParticipantIsConnected` |                 boolean                 | Flag indicating if the local participant is connected and able to publish tracks. |

---

#### useLocalMedia

Provides local media stream capabilities. Primarily used for creating join page experiences. The local audio/video
streams are completely separate from the tracks published in an active session.

Parameters:

|        Parameter        |  Type   | Description                                                     |
|:-----------------------:|:-------:|-----------------------------------------------------------------|
| `videoEnabledByDefault` | boolean | Optional. If true, starts the local video stream automatically. |
| `audioEnabledByDefault` | boolean | Optional. If true, starts the local audio stream automatically. |

Returns:

|         Property         |                  Type                  | Description                                                    |
|:------------------------:|:--------------------------------------:|----------------------------------------------------------------|
|  `currentAudioDeviceId`  |           string &#124; null           | The currently selected audio device, if any.                   |
|  `currentVideoDeviceId`  |           string &#124; null           | The currently selected video device, if any.                   |
|    `localAudioStream`    |   LocalMediaStream &#124; undefined    | The currently active local audio stream, if there is one.      |
|    `localVideoStream`    |   LocalMediaStream &#124; undefined    | The currently active local video stream, if there is one.      |
|  `videoStreamIsLoading`  |                boolean                 | True while the local video stream is loading, false otherwise. |
| `toggleLocalAudioStream` |               () => void               | Turns on or off the currently selected local audio stream.     |
| `toggleLocalVideoStream` |               () => void               | Turns on or off the currently selected local video stream.     |
|   `selectAudioDevice`    | (audioDevice: MediaDeviceInfo) => void | Selects a new audio device for the local audio stream.         |
|   `selectVideoDevice`    | (videoDevice: MediaDeviceInfo) => void | Selects a new video device for the local video stream.         |

---

#### useParticipant

Provides various information primarily for displaying a participant. Can be either the local or a remote participant.

Parameters:

|   Parameter   |                 Type                  | Description                                |
|:-------------:|:-------------------------------------:|--------------------------------------------|
| `participant` | [ParticipantModel](#participantmodel) | The participant model to derive data from. |

Returns:

|       Property        |                      Type                       | Description                                                                                                                      |
|:---------------------:|:-----------------------------------------------:|----------------------------------------------------------------------------------------------------------------------------------|
|         `id`          |               string &#124; null                | Participant identifier.                                                                                                          |
|        `name`         |               string &#124; null                | Participant name.                                                                                                                |
|       `tracks`        | [ParticipantTracksView](#participanttracksview) | Tracks of the participant: all audio and video tracks; the primary audio and video tracks; the screen share track (if available) |
|       `isLocal`       |                     boolean                     | Flag indicating if the participant is the local participant or not.                                                              |
|     `isSpeaking`      |                     boolean                     | Flag indicating if the participant is currently speaking or not.                                                                 |
|   `isScreenSharing`   |                     boolean                     | Flag indicating if the participant is currently sharing their screen or not.                                                     |
|  `connectionQuality`  |                ConnectionQuality                | Connection quality of the participant.                                                                                           |

---

#### useLeaveOnUnload

Convenience hook for leaving the session when the browser window is closed.

Parameters:

| Parameter |    Type    | Description                                                                                                         |
|:---------:|:----------:|---------------------------------------------------------------------------------------------------------------------|
|  `leave`  | () => void | The leave action from [ZocksSessionActions](#zockssessionactions), provided by [useZocksActions](#usezocksactions). |

### Selectors

Hooks usually provide higher level data and functions which can be used without additional configuration. The SDK
also provides a more direct way to access state via selectors. Most of the SDK components use these selectors in
conjunction with the hooks.

Selectors can be used via the `useZocksStore` hook.

Session related selectors:

|                  Selector                   |                    Return type                    | Description                                                                                                                                                                           |
|:-------------------------------------------:|:-------------------------------------------------:|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
|              `selectSessionId`              |                string &#124; null                 | The current session identifier.                                                                                                                                                       |
|              `selectRoomState`              |                     RoomState                     | The current room state.                                                                                                                                                               |
|            `selectIsConnecting`             |                      boolean                      | Flag indicating if a connection is in progress or not.                                                                                                                                |
|          `selectLocalParticipant`           |       [ParticipantModel](#participantmodel)       | The local participant.                                                                                                                                                                |
|         `selectRemoteParticipants`          |      [ParticipantModel](#participantmodel)[]      | The remote participants.                                                                                                                                                              |
|     `selectLocalParticipantIsConnected`     |                      boolean                      | Flag indicating if the local participant is connected or not.                                                                                                                         |
|        `selectTracksForParticipant`         |  [ParticipantTracksView](#participanttracksview)  | The media tracks of a participant. Requires a [ParticipantModel](#participantmodel) parameter. Usage: `useZocksStore((state) => selectTracksForParticipant(state, participantModel))` |
|           `selectIsScreenSharing`           |                      boolean                      | True if someone is sharing their screen, false otherwise.                                                                                                                             |
|      `selectScreenSharingParticipant`       | [ParticipantModel](#participantmodel) &#124; null | The participant who is currently sharing their screen.                                                                                                                                |
|           `selectActiveSpeakers`            |                     string[]                      | Identifiers of the participants who are currently speaking.                                                                                                                           |
|            `selectLastSpeakers`             |                     string[]                      | Identifiers of the participants who were speaking in the past, ordered by the last time they were active.                                                                             |
| `selectUnmutedRemoteParticipantAudioTracks` |             [TrackView](#trackview)[]             | All subscribed and unmuted audio tracks.                                                                                                                                              |

Media device related selectors:

|           Selector           |    Return type     | Description                                                                                    |
|:----------------------------:|:------------------:|------------------------------------------------------------------------------------------------|
| `selectCurrentAudioDeviceId` | string &#124; null | The currently selected audio device id.                                                        |
| `selectCurrentVideoDeviceId` | string &#124; null | The currently selected video device id.                                                        |
| `selectCurrentMediaDeviceId` | string &#124; null | Has a `MediaDeviceKind` parameter. Returns the currently selected device id based on the kind. |

Virtual background related selectors:

|             Selector             |            Return type            | Description                     |
|:--------------------------------:|:---------------------------------:|---------------------------------|
| `selectCurrentVirtualBackground` | VirtualBackgroundType &#124; null | The current virtual background. |

Transcription related selectors:

|               Selector                |                                     Return type                                     | Description                                                               |
|:-------------------------------------:|:-----------------------------------------------------------------------------------:|---------------------------------------------------------------------------|
| `selectTranscriptionServiceIsEnabled` |                                       boolean                                       | True if the transcription service is enabled, false otherwise.            |
| `selectTranscriptionServiceIsLoading` |                                       boolean                                       | True if the transcription service is loading, false otherwise.            |
|    `selectTranscriptionInProgress`    |                                       boolean                                       | True if the local transcription is in progress, false otherwise.          |
|          `selectTranscripts`          | {[participantId: string]: [ParticipantTranscriptModel](#participanttranscriptmodel) | Transcriptions of all participants.                                       |
|    `selectParticipantTranscripts`     |              [ParticipantTranscriptModel](#participanttranscriptmodel)              | Has a `participantId` parameter. Transcriptions of the given participant. |

### Models

#### ZocksConfig

|      Property      |        Type         | Description                                                   |
|:------------------:|:-------------------:|---------------------------------------------------------------|
|    `accountId`     |       string        | Zocks account identifier.                                     |
|    `apiBaseUrl`    |       string?       | Optional. Zocks API URL. Default is `https://api.zocks.io/v1` |
| `apiTokenProvider` | ApiTokenProvider[]? | Optional. API token providers. See the Zocks Javascript SDK.  |
| `apiTokenKeyName`  |       string?       | Optional. API token key name. See the Zocks Javascript SDK.   |
|     `logLevel`     |       string?       | Optional. Zocks log level. See the Zocks Javascript SDK.      |

#### ZocksActions

|      Property       |                    Type                     | Description                                    |
|:-------------------:|:-------------------------------------------:|------------------------------------------------|
|  `sessionActions`   | [ZocksSessionActions](#zockssessionactions) | Session related actions.                       |
| `localMediaActions` |   [ZocksMediaActions](#zocksmediaactions)   | Local media related actions.                   |
|    `chatActions`    |    [ZocksChatActions](#zockschatactions)    | Chat related actions.                          |
|      `session`      |          Session &#124; undefined           | The current session (from the Javascript SDK). |

#### ZocksSessionActions

|   Property   |                        Type                        | Description                                                                      |
|:------------:|:--------------------------------------------------:|----------------------------------------------------------------------------------|
|    `join`    | (joinOptions: [JoinOptions](#joinoptions)) => void | Joins a session by providing a room identifier an optionally additional options. |
|   `leave`    |                     () => void                     | Leaves the session.                                                              |
| `createRoom` |              () => Promise\<RoomInfo>              | Creates a room for future use. Returns information about the created room.       |

#### JoinOptions

|         Property          |           Type           | Description                                                                                                  |
|:-------------------------:|:------------------------:|--------------------------------------------------------------------------------------------------------------|
|         `roomId`          |          string          | The room identifier to join to.                                                                              |
|       `autoCreate`        |         boolean?         | Optional. If the room should be automatically created or not. Default `false`                                |
|      `autoSubscribe`      |         boolean?         | Optional. If `true`, the client will automatically subscribe to published tracks. Default `true`             |
|   `audioDeviceSettings`   |   MediaTrackSettings?    | Optional. If provided, the audio device identified by the settings is automatically published after joining. |
|   `videoDeviceSettings`   |   MediaTrackSettings?    | Optional. If provided, the video device identified by the settings is automatically published after joining. |
|    `virtualBackground`    |  VirtualBackgroundType?  | Optional. If provided, the virtual background will be automatically enabled after joining.                   |
| `virtualBackgroundConfig` | VirtualBackgroundConfig? | Optional. If provided, the virtual background will be enabled with this configuration after joining.         |
|    `afterJoinCallback`    |       () => void?        | Optional. Utility callback function which will be called immediately after successfully joining              |
|    `joinErrorCallback`    |   (e: Error) => void?    | Optional. Utility callback function which will be called with the error as a parameter if joining fails      |

#### ZocksMediaActions

|         Property         |                                                                Type                                                                | Description                                                                                                         |
|:------------------------:|:----------------------------------------------------------------------------------------------------------------------------------:|---------------------------------------------------------------------------------------------------------------------|
|      `publishVideo`      |                                         (mediaTrackSettings?: MediaTrackSettings) => void                                          | Publishes a local video track. Optional `mediaTrackSettings` parameter can be used for specifying the video device. |
|      `publishAudio`      |                                         (mediaTrackSettings?: MediaTrackSettings) => void                                          | Publishes a local audio track. Optional `mediaTrackSettings` parameter can be used for specifying the audio device. |
|   `publishScreenShare`   |                                                             () => void                                                             | Initiates publishing a local screen share track.                                                                    |
|     `unpublishVideo`     |                                         (mediaTrackSettings?: MediaTrackSettings) => void                                          | Unpublishes a local video track.                                                                                    |
|     `unpublishAudio`     |                                         (mediaTrackSettings?: MediaTrackSettings) => void                                          | Unpublishes a local audio track.                                                                                    |
|  `unpublishScreenShare`  |                                                             () => void                                                             | Unpublishes the local screen share track.                                                                           |
|   `switchActiveDevice`   |                                         (kind: MediaDeviceKind, deviceId: string) => void                                          | Switches an audio/video device based on the `kind` and `deviceId` parameters.                                       |
|  `setVirtualBackground`  | (type: VirtualBackgroundType, config?: VirtualBackgroundConfig) => Promise\<void>(kind: MediaDeviceKind, deviceId: string) => void | Sets a virtual background.                                                                                          |
| `clearVirtualBackground` |                                                        () => Promise\<void>                                                        | Clears the current virtual background.                                                                              |

#### ZocksTranscriptionActions

|        Property        |                             Type                             | Description                                                                                    |
|:----------------------:|:------------------------------------------------------------:|------------------------------------------------------------------------------------------------|
| `enableTranscription`  |       (config?: TranscriptionConfig) => Promise\<void>       | Enables the transcription service with the specified config.                                   |
| `disableTranscription` |                     () => Promise\<void>                     | Disables the transcription service.                                                            |
|  `startTranscription`  | (audioConstraints?: MediaTrackConstraints) => Promise\<void> | Starts the transcription. If given, uses the audio device specified by the `audioConstraints`. |
|  `stopTranscription`   |                     () => Promise\<void>                     | Stops the transcription.                                                                       |

#### TrackModel

|    Property    |  Type   | Description                                         |
|:--------------:|:-------:|-----------------------------------------------------|
|      `id`      | string  | Track identifier.                                   |
|   `isMuted`    | boolean | Flag indicating the muted status of the track.      |
| `isSubscribed` | boolean | Flag indicating the subscribed status of the track. |

#### ParticipantTracksModel

|      Property      |                 Type                  | Description                                                         |
|:------------------:|:-------------------------------------:|---------------------------------------------------------------------|
|   `videoTracks`    |      [TrackModel](#trackmodel)[]      | Video tracks of the participant.                                    |
|   `audioTracks`    |      [TrackModel](#trackmodel)[]      | Audio tracks of the participant.                                    |
| `screenShareTrack` | [TrackModel](#trackmodel) &#124; null | Screen share track of the participant if available, null otherwise. |

#### ParticipantModel

|      Property       |                       Type                        | Description                      |
|:-------------------:|:-------------------------------------------------:|----------------------------------|
|        `id`         |                string &#124; null                 | Participant identifier.          |
|       `name`        |                string &#124; null                 | Participant name.                |
|      `tracks`       | [ParticipantTracksModel](#participanttracksmodel) | Track models of the participant. |
| `connectionQuality` |                 ConnectionQuality                 | Participant connection quality.  |

#### SessionModel

|       Property       |                     Type                     | Description                                                                       |
|:--------------------:|:--------------------------------------------:|-----------------------------------------------------------------------------------|
|     `sessionId`      |              string &#124; null              | Session identifier.                                                               |
|     `roomState`      |                  RoomState                   | Room state.                                                                       |
|    `isConnecting`    |                   boolean                    | Flag indicating if a room connection is in progress or not.                       |
|  `localParticipant`  |    [ParticipantModel](#participantmodel)     | Local participant.                                                                |
| `remoteParticipants` |   [ParticipantModel](#participantmodel)[]    | Remote participants.                                                              |
|      `speakers`      | {activeSpeakers: string[], lastSpeakers: []} | Current and last active speakers for speaker indication and participant ordering. |

Views are primarily used for display purposes and contain derived data from the state. The most important
is [ParticipantView](#participantview), which contains information necessary for displaying a single participant,
including their media
tracks, since tracks themselves are not stored in the state.

#### TrackView

|    Property    |  Type   | Description                                        |
|:--------------:|:-------:|----------------------------------------------------|
|    `track`     |  Track  | The media track which can be attached to elements. |
|   `isMuted`    | boolean | Flag indicating the muted status of the track.     |
| `isSubscribed` | boolean | Flag indicating the muted subscribed of the track. |

#### ParticipantTracksView

|      Property       |                Type                 | Description                                                                                                                                                                                         |
|:-------------------:|:-----------------------------------:|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
|    `videoTracks`    |      [TrackView](#trackview)[]      | All available video tracks of the participant.                                                                                                                                                      |
|    `audioTracks`    |      [TrackView](#trackview)[]      | All available audio tracks of the participant.                                                                                                                                                      |
| `primaryVideoTrack` | [TrackView](#trackview) &#124; null | The first subscribed and unmuted video track of the participant. If there is no such video track, then the first video track of the participant. If the participant has no video tracks, then null. |
| `primaryAudioTrack` | [TrackView](#trackview) &#124; null | The first subscribed and unmuted audio track of the participant. If there is no such audio track, then the first audio track of the participant. If the participant has no audio tracks, then null. |
| `screenShareTrack`  | [TrackView](#trackview) &#124; null | The screen share track of the participant if available, null otherwise.                                                                                                                             |

#### ParticipantView

|      Property       |                      Type                       | Description                                                                  |
|:-------------------:|:-----------------------------------------------:|------------------------------------------------------------------------------|
|        `id`         |               string &#124; null                | Participant identifier.                                                      |
|       `name`        |               string &#124; null                | Participant name.                                                            |
|      `tracks`       | [ParticipantTracksView](#participanttracksview) | Tracks of the participant.                                                   |
|      `isLocal`      |                     boolean                     | Flag indicating if the participant is the local participant or not.          |
|    `isSpeaking`     |                     boolean                     | Flag indicating if the participant is currently speaking or not.             |
|  `isScreenSharing`  |                     boolean                     | Flag indicating if the participant is currently sharing their screen or not. |
| `connectionQuality` |                ConnectionQuality                | Connection quality of the participant.                                       |

#### ParticipantTranscriptModel

|      Property       |              Type               | Description                                                          |
|:-------------------:|:-------------------------------:|----------------------------------------------------------------------|
| `finalTranscripts`  |      TranscriptionResult[]      | All final transcriptions results of the participant.                 |
| `partialTranscript` | TranscriptionResult &#124; null | The current partial transcription result of the participant, if any. |


## Chat-related hooks

#### useChat

Convenience hook for all chat related actions and data.

Parameters:

|   Parameter   |                 Type                  | Description                                                                                           |
|:-------------:|:-------------------------------------:|-------------------------------------------------------------------------------------------------------|
| `chatActions` | [ZocksChatActions](#zockschatactions) | The chat actions from [ZocksActions](#ZocksActions), provided by [useZocksActions](#usezocksactions). |

Returns:

|        Property         |                                                                     Type                                                                     | Description                                                                                                             |
|:-----------------------:|:--------------------------------------------------------------------------------------------------------------------------------------------:|-------------------------------------------------------------------------------------------------------------------------|
|   `sendPublicMessage`   |                             (message: string, attachment?: [ChatAttachmentModel](#chatattachmentmodel)) => void                              | Sends a public chat message, optionally with an attachment.                                                             |
|  `sendPrivateMessage`   | (message: string, remoteParticipant: [ParticipantModel](#participantmodel), attachment?: [ChatAttachmentModel](#chatattachmentmodel) => void | Sends a private chat message to the specified remote participant, optionally with an attachment.                        |
|   `uploadAttachment`    |                 (file: File, remoteParticipant?: [ParticipantModel](#participantmodel)) => Promise\<string &#124; undefined>                 | Uploads a file attachment, optionally addressed to a remote participant.                                                |
| `getAttachmentLocation` |                                       (attachmentFileId: string) => Promise\<string &#124; undefined>                                        | Provides a download link for a previously uploaded attachment file.                                                     |
|  `publicChatMessages`   |                                                   [ChatMessageModel](#chatmessagemodel)[]                                                    | All public chat messages.                                                                                               |
|  `privateChatMessages`  |                                      {[participantId: string]: [ChatMessageModel](#chatmessagemodel)[]}                                      | Remote participant → chat messages mapping for private chat messages.                                                   |
|   `messageRecipient`    |                                                                    string                                                                    | Currently selected message recipient. Public message recipients are identified by the `PUBLIC_CHAT_RECIPIENT` constant. |

---

#### useNewMessagesFrom

Hook for accessing part of the state which stores remote participants with unread messages for display purposes.

|      Property      |             Type              | Description                                                                                    |
|:------------------:|:-----------------------------:|------------------------------------------------------------------------------------------------|
| `newMessagesFrom`  |           string[]            | Remote participant identifiers with unread messages. Can also contain `PUBLIC_CHAT_RECIPIENT`. |
| `clearMessageFrom` | (messageFrom: string) => void | Remove the given sender from the unread messages list, clearing it from `newMessagesFrom`.     |

### Selectors

Similarly to session related selectors, chat selectors can be used via the `useZocksStore` hook.

|          Selector           |                             Return type                              | Description                                                                                    |
|:---------------------------:|:--------------------------------------------------------------------:|------------------------------------------------------------------------------------------------|
| `selectPublicChatMessages`  |               [ChatMessageModel](#chatmessagemodel)[]                | Public chat messages.                                                                          |
| `selectPrivateChatMessages` | { [participantId: string]: [ChatMessageModel](#chatmessagemodel)[] } | Private chat messages.                                                                         |
|  `selectMessageRecipient`   |                                string                                | Current message recipient.                                                                     |
|   `selectNewMessagesFrom`   |                               string[]                               | Remote participant identifiers with unread messages. Can also contain `PUBLIC_CHAT_RECIPIENT`. |

### Models

#### ZocksChatActions

|        Property         |                                                                               Type                                                                               | Description                                                                                                             |
|:-----------------------:|:----------------------------------------------------------------------------------------------------------------------------------------------------------------:|-------------------------------------------------------------------------------------------------------------------------|
|   `sendPublicMessage`   |                                       (message: string, attachment?: [ChatAttachmentModel](#chatattachmentmodel)) => void                                        | Sends a public chat message, optionally with an attachment.                                                             |
|  `sendPrivateMessage`   | (message: string, remoteParticipant: [ParticipantModel](#participantmodel), attachment?: ([ChatAttachmentModel](#chatattachmentmodel) &#124; undefined)) => void | Sends a private chat message to the specified remote participant, optionally with an attachment.                        |
|   `uploadAttachment`    |                           (file: File, remoteParticipant?: [ParticipantModel](#participantmodel)) => Promise\<string &#124; undefined>                           | Uploads a file attachment, optionally addressed to a remote participant.                                                |
| `getAttachmentLocation` |                                                 (attachmentFileId: string) => Promise\<string &#124; undefined>                                                  | Provides a download link for a previously uploaded attachment file.                                                     |

#### ChatMessageModel

|   Property   |                     Type                     | Description                                                             |
|:------------:|:--------------------------------------------:|-------------------------------------------------------------------------|
|    `type`    |               ChatMessageType                | Type of the chat message. `PUBLIC` or `PRIVATE`.                        |
|  `message`   |                    string                    | The chat message.                                                       |
| `timestamp`  |                    number                    | Epoch timestamp in seconds, indicates send time.                        |
|    `from`    |          {id: string, name: string}          | Sender identifier and name.                                             |
|     `to`     |         {id: string, name: string}?          | Optional. Recipient identifier and name. Not given for public messages. |
| `attachment` | [ChatAttachmentModel](#chatattachmentmodel)? | Optional. The attachment of the chat message.                           |

#### ChatAttachmentModel

|   Property    |  Type  | Description                                                          |
|:-------------:|:------:|----------------------------------------------------------------------|
|   `fileId`    | string | Identifier of the attached file. Used for generating a download URL. |
|  `fileName`   | string | Name of the attached file.                                           |
|  `fileSize`   | number | File size in bytes.                                                  |
| `contentType` | string | Content type of the attached file.                                   |

There are several other hooks for specialized uses cases in components. Those are described with their corresponding
components.

## Components

### Video

Renders a `video` element from a video stream.

_Props_:

|   Property    |              Type               | Description                 |
|:-------------:|:-------------------------------:|-----------------------------|
| `videoStream` | DOMAttachableStream &#124; null | The video stream to render. |

_Notes_:

* Both tracks provided by [TrackViews](#trackview) and local media streams are compatible with the component.
* Uses the custom `useVideo` hook, which handles automatic attachment and detachment of tracks, and provides a `ref`
  for the video element.

---

### Audio

Renders an `audio` element from an audio stream.

_Props_:

|   Property    |              Type               | Description                 |
|:-------------:|:-------------------------------:|-----------------------------|
| `audioStream` | DOMAttachableStream &#124; null | The audio stream to render. |

_Notes_:

* Both tracks provided by [TrackViews](#trackview) and local media streams are compatible with the component.
* Uses the custom `useAudio` hook, which handles automatic attachment and deattachment of tracks, and provides a `ref`
  for the audio element.

---

### Participant

Renders a single participant. Renders the participant video if available, and optionally the participant audio if
available. Also renders status icons for audio, video, and connection quality.

The component can be optionally extended by a custom component, enriching its functionality.

_Props_ (`ParticipantProps`):

|      Property      |                 Type                  | Description                                                                              |
|:------------------:|:-------------------------------------:|------------------------------------------------------------------------------------------|
|   `participant`    | [ParticipantModel](#participantmodel) | The participant model                                                                    |
|    `withAudio`     |               boolean?                | Optional. If true, renders also participant audio.                                       |
| `extensionElement` | React.FC\<ParticipantExtensionProps>? | Optional. If given, also renders the component with the props derived from `participant` |

_Extension props_ (`ParticipantExtensionProps`):

|      Property       |        Type        | Description                                                       |
|:-------------------:|:------------------:|-------------------------------------------------------------------|
|   `participantId`   | string &#124; null | Participant identifier                                            |
|  `participantName`  | string &#124; null | Participant name                                                  |
|      `isLocal`      |      boolean       | True if the participant is the local participant, false otherwise |
| `connectionQuality` | ConnectionQuality  | Connection quality                                                |
|    `isSpeaking`     |      boolean       | True if the participant is currently speaking, false otherwise    |
|   `videoIsMuted`    |      boolean       | True if the participant does not have video                       |
|   `audioIsMuted`    |      boolean       | True if the participant does not have audio                       |
|  `isScreenSharing`  |      boolean       | True if the participant is sharing their screen                   |

_Related CSS classes_:

- `participant`
- `participant__name`
- `participant__video-container`
- `participant__video-container--speaking`
- `participant__video-container--local`
- `participant__avatar`
- `participant__status-icons`
- `participant__no-video-icon`
- `participant__no-audio-icon`
- `participant__unknown-quality-icon`
- `participant__poor-quality-icon`
- `participant__good-quality-icon`
- `participant__excellent-quality-icon`

---

### ScreenShare

Simple component rendering a screen share video.

_Props_ (`ScreenShareProps`):

|   Property    |                 Type                  | Description           |
|:-------------:|:-------------------------------------:|-----------------------|
| `participant` | [ParticipantModel](#participantmodel) | The participant model |

_Related CSS classes_:

- `screen-share`

---

### ParticipantsGridView

Renders all participants in a grid, with paging. The grid size changes based on the number of displayed participants.
Currently, the maximum grid is 5x5.

By default, participants are rendered with the `Participant` component, but the participant renderer can be changed.

_Props_ (`MultipleParticipantViewProps`):

|       Property       |             Type             | Description                                                                        |
|:--------------------:|:----------------------------:|------------------------------------------------------------------------------------|
|      `pageSize`      |           number?            | Optional. The maximum number of participants on a single page. Default `25`.       |
| `participantElement` | React.FC\<ParticipantProps>? | Optional. The component to render single participants with. Default `Participant`. |

_Related CSS classes_:

- `participants-grid-view`
- `participants-grid-view__grid`
- `participants-grid-view__button`
- `participants-grid-view__previous-button`
- `participants-grid-view__previous-button-icon`
- `participants-grid-view__next-button`
- `participants-grid-view__next-button-icon`

---

### ParticipantsSpeakerView

Renders the main speaker participant (the participant who is currently speaking, or was the last one speaking), and all
other participants on the side, with paging.

By default, participants are rendered with the `Participant` component, but the participant renderer can be changed.

_Props_ (`MultipleParticipantViewProps`):

|       Property       |             Type             | Description                                                                        |
|:--------------------:|:----------------------------:|------------------------------------------------------------------------------------|
|      `pageSize`      |           number?            | Optional. The maximum number of participants on a single page. Default `4`.        |
| `participantElement` | React.FC\<ParticipantProps>? | Optional. The component to render single participants with. Default `Participant`. |

_Related CSS classes_:

- `participants-speaker-view`
- `participants-speaker-view__active-speaker`
- `participants-speaker-view__only-speaker`
- `participants-speaker-view__others-container`
- `participants-speaker-view__others`
- `participants-speaker-view__button`
- `participants-speaker-view__previous-button`
- `participants-speaker-view__previous-button-icon`
- `participants-speaker-view__next-button`
- `participants-speaker-view__next-button-icon`

---

### ParticipantsScreenShareView

Renders a shared screen, and all participants on the side, with paging.

By default, participants are rendered with the `Participant` component, and the screen share with the `ScreenShare`
component,
but the these renderers can be changed.

_Props_ (`ScreenShareViewProps`):

|       Property       |             Type             | Description                                                                        |
|:--------------------:|:----------------------------:|------------------------------------------------------------------------------------|
|      `pageSize`      |           number?            | Optional. The maximum number of participants on a single page. Default `4`.        |
| `participantElement` | React.FC\<ParticipantProps>? | Optional. The component to render single participants with. Default `Participant`. |
| `screenShareElement` | React.FC\<ParticipantProps>? | Optional. The component to render the screen share. Default `ScreenShare`.         |

_Related CSS classes_:

- `participants-screen-share-view`
- `participants-screen-share-view__current-share`
- `participants-screen-share-view__participants-container`
- `participants-screen-share-view__participants`
- `participants-screen-share-view__button`
- `participants-screen-share-view__previous-button`
- `participants-screen-share-view__previous-button-icon`
- `participants-screen-share-view__next-button`
- `participants-screen-share-view__next-button-icon`

---

### OneOnOneParticipantView

Renders a picture in picture participant view for one on one meetings with a built-in join page.
By default, participants are rendered with the `Participant` component, and the screen share with the `ScreenShare`
component, but the these renderers can be changed.

This component can be embedded into any page and renders a full screen button which puts the whole component above
everything.

_Props_ (`OneOnOneParticipantView`):

|             Property              |                   Type                    | Description                                                                                                                                       |
|:---------------------------------:|:-----------------------------------------:|---------------------------------------------------------------------------------------------------------------------------------------------------|
|          `zocksActions`           |               ZocksActions                | Required. The result of the `useZocksActions`                                                                                                     |
|             `roomId`              |                  string                   | Required. The ID of the room where the component should connect.                                                                                  |
|        `emptyJoinPageText`        |                  string?                  | Optional. This text will be rendered when camera is turned off on the join page. Default: `Your camera is turned off.`                            |
|      `participantNameHidden`      |                 boolean?                  | Optional. Renders the remote participant name if this is set to `false`. Default `true`.                                                          |
|   `emptyRemoteParticipantText`    |                  string?                  | Optional. This text will be rendered when there is no remote participant in the connected room. Default `Waiting for other participant to join.`. |
|        `joinButtonElement`        |        React.FC\<JoinButtonProps>?        | Optional. The component to render join button on the join page. Default `JoinButton`.                                                             |
|        `joinLoaderElement`        |                 React.FC?                 | Optional. The component to render when joining to the room. Default `Loader`.                                                                     |
|     `localParticipantElement`     |       React.FC\<ParticipantProps>?        | Optional. The component to render the local participant. Default `Participant`.                                                                   |
|    `remoteParticipantElement`     |       React.FC\<ParticipantProps>?        | Optional. The component to render the remote participant. Default `Participant`.                                                                  |
|    `audioControlButtonElement`    |    React.FC\<AudioControlButtonProps>?    | Optional. The component to render the audio control button. Default `CustomAudioControlButton`.                                                   |
|    `videoControlButtonElement`    |    React.FC\<VideoControlButtonProps>?    | Optional. The component to render the video control button. Default `CustomVideoControlButton`.                                                   |
| `screenShareControlButtonElement` | React.FC\<ScreenShareControlButtonProps>? | Optional. The component to render the screen share. Default `ScreenShare`.                                                                        |
|       `leaveButtonElement`        |       React.FC\<LeaveButtonProps>?        | Optional. The component to render the leave button. Default `LeaveButton`.                                                                        |
|     `fullScreenButtonElement`     |     React.FC\<FullScreenButtonProps>?     | Optional. The component to render the full screen button. Default `FullScreenButton`.                                                             |

_Related CSS classes_:

- `join-page-in-video__local-stream-buttons`
- `join-page-in-video__join-button`
- `join-page-in-video__no-video-icon`
- `oneonone-participant-container`
- `oneonone-participants-container`
- `oneonone-participants-view--full-screen`
    - common issue some of your own component overlays fulls screen: add `z-index` css to overlay those component.
- `oneonone-participants-view__active-speaker`
- `oneonone-participants-view__local-participant`
- `oneonone-participants-view__room-controls`
-

---

### ViewSwitcher

Simple component for switching between grid and speaker views. The current view is stored in state, so it can be changed
globally.

_Props_: none

_Notes_:

Has a custom hook, `useViewSwitcher`, which returns the current view (`currentView` property), and a function to change
views (`changeView` property). Possible views are `ParticipantsViewType.Grid` and `ParticipantsViewType.Speaker`

---

### ParticipantAudio

Renders `audio` elements for all unmuted remote participants. Useful for views displaying all participants, so no audio
track has to be individually rendered.

_Props_: none

_Notes_:

Uses the `selectUnmutedRemoteParticipantAudioTracks` selector for retrieving all audio tracks.

---

### Participants

Composite component displaying all participants in the currently selected view, and a view switcher. If a screen share
is
going on, then displays the screen share.

_Props_:

|       Property        |             Type             | Description                                                                                      |
|:---------------------:|:----------------------------:|--------------------------------------------------------------------------------------------------|
|    `gridPageSize`     |           number?            | Optional. The maximum number of participants on a single page in grid view. Default `25`.        |
|   `speakerPageSize`   |           number?            | Optional. The maximum number of participants on a single page in speaker view. Default `4`.      |
| `screenSharePageSize` |           number?            | Optional. The maximum number of participants on a single page in screen share view. Default `4`. |
| `participantElement`  | React.FC\<ParticipantProps>? | Optional. The component to render single participants with. Default `Participant`.               |
| `screenShareElement`  | React.FC\<ParticipantProps>? | Optional. The component to render the screen share. Default `ScreenShare`.                       |

---

### ParticipantsList

Renders all participants in a dismissible list. Usually used for displaying the complete list of participants in a room,
optionally
with their media status.

_Props_ (`ParticipantsListProps`):

|       Property       |                  Type                   | Description                                                                                |
|:--------------------:|:---------------------------------------:|--------------------------------------------------------------------------------------------|
|    `participants`    | [ParticipantModel](#participantmodel)[] | The participants to display.                                                               |
|      `dismiss`       |               () => void?               | Optional. The function to call when dismissing the list.                                   |
| `participantElement` | React.FC\<ParticipantsListElementProps> | Optional. The participant list element to render. Default: `SimpleParticipantsListElement` |

_List element props_ (`ParticipantsListElementProps`):

|   Property    |                 Type                  | Description                 |
|:-------------:|:-------------------------------------:|-----------------------------|
| `participant` | [ParticipantModel](#participantmodel) | The participant to display. |

_Related CSS classes_:

- `participants-list`
- `participants-list__participants-container`
- `participants-list__dismiss-button`
- `participants-list__participant-container`
- `participants-list__participant-avatar`
- `participants-list__participant-name`

_Notes_:

Used by `OnlineParticipantsList`.

---

### OnlineParticipantsList

Extended version of `ParticipantsList` via using a custom participant list element renderer. Renders all online
participants
with their status (whether they have audio and video, and whether they are speaking or not).

_Props_ (`OnlineParticipantsListProps`):

|       Property       |                  Type                   | Description                                                                                |
|:--------------------:|:---------------------------------------:|--------------------------------------------------------------------------------------------|
|      `dismiss`       |               () => void?               | Optional. The function to call when dismissing the list.                                   |
| `participantElement` | React.FC\<ParticipantsListElementProps> | Optional. The participant list element to render. Default: `SimpleParticipantsListElement` |

---

### JoinPage

Provides a complete join page experience with switchable audio and video streams, ability to join an existing meeting,
create a new meeting now or create a new meeting for later.

_Props_:

|        Property         |                        Type                        | Description                                                                                              |
|:-----------------------:|:--------------------------------------------------:|----------------------------------------------------------------------------------------------------------|
|      `joinAction`       | (joinOptions: [JoinOptions](#joinoptions)) => void | Action to call when clicking the Join or New Meeting buttons.                                            |
|   `createRoomAction`    |              () => Promise\<RoomInfo>              | Action to call when clicking the Create New Meeting button.                                              |
|     `initialRoomId`     |                      string?                       | Optional. The initial value of the room input.                                                           |
|    `enableAudioText`    |                      string?                       | Optional. Text of the Enable Audio button.                                                               |
|   `disableAudioText`    |                      string?                       | Optional. Text of the Disable Audio button.                                                              |
|    `enableVideoText`    |                      string?                       | Optional. Text of the Enable Video button.                                                               |
|   `disableVideoText`    |                      string?                       | Optional. Text of the Disable Video button.                                                              |
|       `joinText`        |                      string?                       | Optional. Text of the Join button.                                                                       |
|    `newMeetingText`     |                      string?                       | Optional. Text of the New Meeting button.                                                                |
| `createNewMeetingText`  |                      string?                       | Optional. Text of the Create New Meeting button.                                                         |
|  `newMeetingDisabled`   |                      boolean?                      | Optional. Disabled state of the New Meeting button.                                                      |
|   `roomIdInputLabel`    |                      string?                       | Optional. Label of the room input.                                                                       |
| `roomIdInputErrorLabel` |                      string?                       | Optional. Text to display when the room is in error state.                                               |
|  `roomIdInputDisabled`  |                      boolean?                      | Optional. Disabled state of the room input.                                                              |
|   `roomIdPlaceholder`   |                      string?                       | Optional. Placeholder of the room input.                                                                 |
| `videoEnabledByDefault` |                      boolean?                      | Optional. When true, the local video stream is turned on by default.                                     |
| `audioEnabledByDefault` |                      boolean?                      | Optional. When true, the local audio stream is turned on by default.                                     |
|   `onNewRoomCreated`    |           (roomInfo: RoomInfo) => void?            | Optional. Callback when a new room for later is successfully created. Can be used to display room info.. |
|      `joinOptions`      |       Partial\<[JoinOptions](#joinoptions)>?       | Optional. Join options, see the corresponding model.                                                     |

For controlling virtual background, uses the `useLocalVirtualBackgrounds` hook, which provides the following properties
and functions:

|           Property           |                    Type                     | Description                                                                                                      |
|:----------------------------:|:-------------------------------------------:|------------------------------------------------------------------------------------------------------------------|
|    `availableBackgrounds`    |           VirtualBackgroundType[]           | The available virtual backgrounds.                                                                               |
|  `currentVirtualBackground`  |      VirtualBackgroundType &#124; null      | The current virtual background of the local participant, if any.                                                 |
| `virtualBackgroundIsLoading` |                   boolean                   | Flag indicating that the virtual background is loading.                                                          |
|  `virtualBackgroundStream`   |              LocalMediaStream?              | Media stream of the current virtual background, if any.                                                          |
|  `changeVirtualBackground`   | (VirtualBackgroundType &#124; null) => void | Sets or clears the current virtual background.                                                                   |
| `stopLocalVirtualBackground` |                 () => void                  | Stops processing the current virtual background. Can be used for stopping the processing after joining the room. |

_Related CSS classes_:

- `join-page`
- `join-page__local-streams`
- `join-page__video-container`
- `join-page__video-preview`
- `join-page__no-video-icon`
- `join-page__local-stream-buttons`
- `join-page__button`
- `join-page__audio-button-container`
- `join-page__audio-button`
- `join-page__audio-button--enabled`
- `join-page__audio-button--disabled`
- `join-page--audio-select-button`
- `join-page__video-button-container`
- `join-page__video-button`
- `join-page__video-button--enabled`
- `join-page__video-button--disabled`
- `join-page--video-select-button`
- `join-page__controls`
- `join-page__join-room-container`
- `join-page__room-id-input--error`
- `join-page__join-button`
- `join-page__controls-divider`
- `join-page__new-meeting-container`
- `join-page__new-meeting-button`
- `join-page__create-new-meeting-button`

---

### DeviceSelectButton

Button for listing and selecting media devices.

_Props_:

|      Property      |                    Type                    | Description                                |
|:------------------:|:------------------------------------------:|--------------------------------------------|
|    `deviceKind`    |              MediaDeviceKind               | The media device kind.                     |
| `onDeviceSelected` | (mediaDeviceInfo: MediaDeviceInfo) => void | Called when a device is selected.          |
|     `disabled`     |                  boolean?                  | Optional. Disabled state of the button.    |
|    `className`     |                  string?                   | Optional. Additional class for the button. |
|      `label`       |                  string?                   | Optional. Button label.                    |

_Related CSS classes_:

- `select-button__button-container`
- `select-button__button`
- `select-button__button-icon`
- `select-button__popover-container`
- `select-button__popover-list`
- `select-button__popover-item`

---

### RoomControls

Provides media controls and a leave button for the local participant.

_Props_:

|              Property              |                    Type                    | Description                                                                                                                         |
|:----------------------------------:|:------------------------------------------:|-------------------------------------------------------------------------------------------------------------------------------------|
|           `mediaActions`           |  [ZocksMediaActions](#zocksmediaactions)   | Actions for controlling the local media. Usually the `mediaActions` from the [useZocksActions](#usezocksactions) hook               |
|           `leaveAction`            |                 () => void                 | Action to call when clicking the Leave button. Usually the `sessionActions.leave` from the [useZocksActions](#usezocksactions) hook |
| `virtualBackgroundSelectorElement` | React.FC\<VirtualBackgroundSelectorProps>? | Element renderer for selecting virtual backgrounds.                                                                                 |

_Related CSS classes_:

- `room-controls`
- `room-controls__control-button`
- `room-controls__audio-button-container`
- `room-controls__audio-button--unmuted`
- `room-controls__audio-button--muted`
- `room-controls--audio-select-button`
- `room-controls__video-button-container`
- `room-controls__video-button--unmuted`
- `room-controls__video-button--muted`
- `room-controls--video-select-button`
- `room-controls__screen-share-button`
- `room-controls__screen-share-button--share-on`
- `room-controls__screen-share-button--share-off`
- `room-controls__leave-button`

_Notes_:

Uses the custom hook called `useRoomControls` which provides the following properties and functions. Can be used for
easily creating custom room controls.

|       Property       |                  Type                  | Description                                                           |
|:--------------------:|:--------------------------------------:|-----------------------------------------------------------------------|
|     `audioIsOn`      |                boolean                 | Flag indicating that the local audio stream is published and unmuted. |
|     `videoIsOn`      |                boolean                 | Flag indicating that the local video stream is published and unmuted. |
|  `isScreenSharing`   |                boolean                 | Flag indicating that there is a screen share going on in the room.    |
| `isLocalScreenShare` |                boolean                 | Flag indicating that the local screen share stream is published.      |
|    `toggleAudio`     |               () => void               | Toggles the local audio stream.                                       |
|    `toggleVideo`     |               () => void               | Toggles the local video stream.                                       |
| `selectAudioDevice`  | (audioDevice: MediaDeviceInfo) => void | Changes the local audio device.                                       |
| `selectVideoDevice`  | (videoDevice: MediaDeviceInfo) => void | Changes the local video device.                                       |

For controlling virtual background, uses the `useVirtualBackgrounds` hook, which provides the following properties and
functions:

|           Property           |                    Type                     | Description                                                      |
|:----------------------------:|:-------------------------------------------:|------------------------------------------------------------------|
|    `availableBackgrounds`    |           VirtualBackgroundType[]           | The available virtual backgrounds.                               |
|  `currentVirtualBackground`  |      VirtualBackgroundType &#124; null      | The current virtual background of the local participant, if any. |
| `virtualBackgroundIsLoading` |                   boolean                   | Flag indicating that the virtual background is loading.          |
|  `changeVirtualBackground`   | (VirtualBackgroundType &#124; null) => void | Sets or clears the current virtual background.                   |

### VirtualBackgroundSelector

Simple selector for virtual backgrounds. Provides all available virtual backgrounds, and a `None` option for clearing
the background.

_Props_ (`VirtualBackgroundSelectorProps`):

|          Property          |                    Type                     | Description                                    |
|:--------------------------:|:-------------------------------------------:|------------------------------------------------|
|   `availableBackgrounds`   |           VirtualBackgroundType[]           | The available virtual backgrounds.             |
| `currentVirtualBackground` |      VirtualBackgroundType &#124; null      | The current virtual background, if any.        |
| `selectVirtualBackground`  | (VirtualBackgroundType &#124; null) => void | Sets or clears the current virtual background. |
|         `disabled`         |                  boolean?                   | Optional. Disabled property of the selector.   |

_Related CSS classes_:

- `virtual-background-select-container`
- `virtual-background-select-text`
- `virtual-background-select-button`

### TranscriptionDisplay

Displays the transcription of a single participant, with configurable word count and hide delay.
Can be used in a [Participant](#participant) extension.

_Props_:

|    Property     |  Type   | Description                                                                          |
|:---------------:|:-------:|--------------------------------------------------------------------------------------|
| `participantId` | string  | The participant id.                                                                  |
|  `wordsInLine`  | number? | Optional. The number of maximum words per line. Default: `10`                        |
|   `hideDelay`   | number? | Optional. Hides the transcription after the specified delay (in ms). Default: `3000` |

For getting participant transcription data, uses the `useTranscript` hook, which provides the following properties:

|            Property             |              Type               | Description                                                                                                                        |
|:-------------------------------:|:-------------------------------:|------------------------------------------------------------------------------------------------------------------------------------|
| `transcriptionServiceIsEnabled` |             boolean             | True if the transcription service is currently enabled, false otherwise.                                                           |
| `transcriptionServiceIsLoading` |             boolean             | True if the transcription service is currently loading, false otherwise.                                                           |
|       `partialTranscript`       | TranscriptionResult &#124; null | The current partial transcript of the participant, if any. Can be used for displaying partial results differently than final ones. |
|       `finalTranscripts`        |      TranscriptionResult[]      | The final (stable) transcript results of the participant.                                                                          |

_Related CSS classes_:

- `transcription-display__container`
- `transcription-display__line`

## Chat-related components

### Chat input

#### ChatMessageInput

Simple multiline message text area with instant message sending.

_Props_ (`ChatMessageInputElementProps`):

|        Property        |           Type            | Description                                              |
|:----------------------:|:-------------------------:|----------------------------------------------------------|
|  `currentChatMessage`  |          string           | The current chat message.                                |
| `onChatMessageChanged` | (message: string) => void | Callback fired when the message is changed.              |
|   `placeholderText`    |          string?          | Optional. Placeholder for the text area.                 |
|     `sendMessage`      |        () => void?        | Optional. For message sending from within the component. |

_Related CSS classes_:

- `chat-input__message-textarea`

_Notes_:

The text area `ref` is forwarded for manual focusing capability.

---

#### SendButton

Simple button for sending a chat message.

_Props_ (`SendButtonElementProps`):

|   Property   |    Type     | Description                               |
|:------------:|:-----------:|-------------------------------------------|
|  `disabled`  |  boolean?   | Optional. Disabled state of the button.   |
|  `onClick`   | () => void? | Optional. Fired when clicking the button. |
| `buttonText` |   string?   | Optional. Text of the button.             |

_Related CSS classes_:

- `chat-input__send-button`

---

#### RecipientSelector

Component for selecting a chat recipient, including public messages.

_Props_ (`RecipientSelectorElementProps`):

|        Property        |                   Type                   | Description                                                          |
|:----------------------:|:----------------------------------------:|----------------------------------------------------------------------|
| `onRecipientSelected`  |      (recipientId: string) => void       | Called when a new recipient is selected.                             |
|  `remoteParticipants`  | [ParticipantModel](#participantmodel)[]? | Optional. The remote participants.                                   |
|    `selectInfoText`    |                 string?                  | Optional. Additional text for the selector.                          |
| `allParticipantsLabel` |                 string?                  | Optional. Label of the public message recipient. Default: `Everyone` |

_Related CSS classes_:

- `chat-input__recipient-select-container`
- `chat-input__recipient-select`

---

#### EmojiSelector

Emoji selector component with customizable emojis. The emojis are selected from a popover, which is opened and closed
with a button.

_Props_ (`EmojiSelectorElementProps`):

|      Property       |                      Type                       | Description                                                                    |
|:-------------------:|:-----------------------------------------------:|--------------------------------------------------------------------------------|
|   `onSelectEmoji`   |             (emoji: string) => void             | Called when an emoji is selected.                                              |
|      `emojis`       |                    string[]?                    | Optional. The list of emojis. Default:  `['😂', '👍', '🎉', '😃', '👏']`       |
| `openButtonElement` | React.FC\<EmojiSelectorOpenButtonElementProps>? | Optional. Element renderer for customizing the button which opens the popover. |
|  `currentMessage`   |                     string?                     | Optional. The current message. Can be used for additional logic.               |

_Props for the open button element_ (`EmojiSelectorOpenButtonElementProps`):

| Property  |    Type    | Description                        |
|:---------:|:----------:|------------------------------------|
| `onClick` | () => void | Called when the button is clicked. |

_Related CSS classes_:

- `emoji-selector__emoji-container`
- `emoji-selector__emoji-button`
- `emoji-selector__open-button`

---

#### AttachmentUploader

Attachment uploader with loading state and customizable file upload button.

_Props_ (`AttachmentUploaderElementProps`):

|            Property             |                                                     Type                                                     | Description                                                                                                     |
|:-------------------------------:|:------------------------------------------------------------------------------------------------------------:|-----------------------------------------------------------------------------------------------------------------|
|       `uploadAttachment`        | (file: File, remoteParticipant?: [ParticipantModel](#participantmodel)) => Promise\<string &#124; undefined> | The upload attachment actions. Usually comes from `chatActions` provided by [useZocksActions](#usezocksactions) |
|      `attachmentRecipient`      |                                    [ParticipantModel](#participantmodel)?                                    | Optional. The recipient of the attachment, if any.                                                              |
|   `onAttachmentUploadStarted`   |                                            (file: File) => void?                                             | Optional. Called when file upload is started.                                                                   |
|  `onAttachmentUploadFinished`   |                                    (file: File, fileId: string) => void?                                     | Optional. Called when file upload has finished successfully.                                                    |
|   `onAttachmentUploadFailed`    |                                            (file: File) => void?                                             | Optional. Called when file upload finished with an error.                                                       |
| `uploadAttachmentButtonElement` |                                React.FC<UploadAttachmentButtonElementProps>?                                 | Optional. Element renderer for customizing the button which opens the input.                                    |

_Props for the open button element_ (`UploadAttachmentButtonElementProps`):

|  Property  |    Type    | Description                             |
|:----------:|:----------:|-----------------------------------------|
| `onClick`  | () => void | Called when the button is clicked.      |
| `disabled` |  boolean?  | Optional. Disabled state of the button. |
| `hasError` |  boolean?  | Optional. Error state of the button.    |

_Related CSS classes_:

- `attachment-uploader__container`
- `attachment-uploader__file-input`

---

#### AttachmentDisplay

Displays an uploaded attachment file before actually sending the chat message with the attachment. Also, able to delete
the attachment.

_Props_ (`AttachmentDisplayElementProps`):

|            Property            |                     Type                      | Description                                                                            |
|:------------------------------:|:---------------------------------------------:|----------------------------------------------------------------------------------------|
|          `attachment`          | [ChatAttachmentModel](#chatattachmentmodel)?  | Optional. The chat attachment, if any. If there is no attachment, nothing is rendered. |
|      `hasAttachmentError`      |                   boolean?                    | Optional. Flag indicating if there was an error when uploading the attachment.         |
|      `onClearAttachment`       |                  () => void?                  | Optional. Called when clearing the attachment.                                         |
| `clearAttachmentButtonElement` | React.FC\<ClearAttachmentButtonElementProps>? | Optional. Element renderer for customizing the button which clears the attachment.     |

_Props for the clear button element_ (`ClearAttachmentButtonElementProps`):

|  Property  |    Type    | Description                             |
|:----------:|:----------:|-----------------------------------------|
| `onClick`  | () => void | Called when the button is clicked.      |

_Related CSS classes_:

- `attachment-display__container`
- `attachment-display__name`
- `attachment-display__error`

---

#### CustomizableChatInput

Composite component utilizing the different input elements for a complete chat input experience.

_Props_ (`ChatInputElementProps & CustomizableChatInputProps`):

|          Property           |                                                                      Type                                                                      | Description                                                                                       |
|:---------------------------:|:----------------------------------------------------------------------------------------------------------------------------------------------:|---------------------------------------------------------------------------------------------------|
|     `messageRecipient`      |                                                                     string                                                                     | The current recipient.                                                                            |
|    `remoteParticipants`     |                                                    [ParticipantModel](#participantmodel)[]?                                                    | Optional. The remote participants.                                                                |
|    `onRecipientSelected`    |                                                         (recipientId: string) => void?                                                         | Optional. Called when a new recipient is selected.                                                |
|     `sendPublicMessage`     |                              (message: string, attachment?: [ChatAttachmentModel](#chatattachmentmodel)) => void?                              | Optional. Sends a new public message, optionally with an attachment.                              |
|    `sendPrivateMessage`     | (message: string, remoteParticipant: [ParticipantModel](#participantmodel), attachment?: [ChatAttachmentModel](#chatattachmentmodel)) => void? | Optional. Sends a new private private message, optionally with an attachment.                     |
|     `uploadAttachment`      |                  (file: File, remoteParticipant?: [ParticipantModel](#participantmodel)) => Promise<string &#124; undefined>?                  | Optional. Uploads an attachment, optionally with a recipient.                                     |
|  `chatMessageInputElement`  |                                                    React.FC<ChatMessageInputElementProps>?                                                     | Optional. Element renderer for customizing the chat input. Default: `ChatMessageInput`            |
|     `sendButtonElement`     |                                                       React.FC<SendButtonElementProps>?                                                        | Optional. Element renderer for customizing the send button. Default: `SendButton`                 |
| `recipientSelectorElement`  |                                                    React.FC<RecipientSelectorElementProps>?                                                    | Optional. Element renderer for customizing the recipient selector. Default: `RecipientSelector`   |
|   `emojiSelectorElement`    |                                                      React.FC<EmojiSelectorElementProps>?                                                      | Optional. Element renderer for customizing the emoji selector. Default: `EmojiSelector`           |
| `attachmentUploaderElement` |                                                   React.FC<AttachmentUploaderElementProps>?                                                    | Optional. Element renderer for customizing the attachment uploader. Default: `AttachmentUploader` |
| `attachmentDisplayElement`  |                                                    React.FC<AttachmentDisplayElementProps>?                                                    | Optional. Element renderer for customizing the attachment display. Default: `AttachmentDisplay`   |

_Related CSS classes_:

- `chat-input__input-group`
- `chat-input__message-input-container`
- `chat-input__top-bar`
- `chat-input__loader-container`
- `chat-input__loader`

### Chat message

#### ChatMessageWithAvatar

Renders a chat message, optionally with a participant avatar.

_Props_ (`ChatMessageElementProps & AvatarProps`):

|            Property            |                          Type                          | Description                                                                                                                                        |
|:------------------------------:|:------------------------------------------------------:|----------------------------------------------------------------------------------------------------------------------------------------------------|
|         `chatMessage`          |         [ChatMessageModel](#chatmessagemodel)          | The chat message to display.                                                                                                                       |
|          `showAvatar`          |                        boolean                         | Whether the avatar should be displayed or not.                                                                                                     |
|    `getAttachmentLocation`     | (fileId: string) => Promise\<string &#124; undefined>? | Optional. Function for getting a download URL for an attachment. Usually comes from `chatActions` provided by [useZocksActions](#usezocksactions). |
| `chatMessageAttachmentElement` |     React.FC\<ChatMessageAttachmentElementProps>?      | Optional. Element renderer for customizing how the attachment is displayed. See `ChatMessageAttachment`.                                           |

_Related CSS classes_:

- `chat-message-a__container`
- `chat-message-a__container--sent`
- `chat-message-a__container--received`
- `chat-message-a__message-container`
- `chat-message-a__avatar`
- `chat-message-a__message`

_Notes_:

Used in `SplitChat` for a chat experience with separate threads.

---

#### ChatMessageWithParticipantInfo

Renders a chat message, optionally with additional information about the participants, such as their name..

_Props_ (`ChatMessageElementProps & InfoProps`):

|            Property            |                          Type                          | Description                                                                                                                                        |
|:------------------------------:|:------------------------------------------------------:|----------------------------------------------------------------------------------------------------------------------------------------------------|
|         `chatMessage`          |         [ChatMessageModel](#chatmessagemodel)          | The chat message to display.                                                                                                                       |
|     `showParticipantInfo`      |                        boolean                         | Whether additional participant information should be displayed or not.                                                                             |
|    `localParticipantLabel`     |                        string?                         | Optional. Name of the local participant. Default: `Me`                                                                                             |
|    `getAttachmentLocation`     | (fileId: string) => Promise\<string &#124; undefined>? | Optional. Function for getting a download URL for an attachment. Usually comes from `chatActions` provided by [useZocksActions](#usezocksactions). |
| `chatMessageAttachmentElement` |     React.FC\<ChatMessageAttachmentElementProps>?      | Optional. Element renderer for customizing how the attachment is displayed. See `ChatMessageAttachment`.                                           |

_Related CSS classes_:

- `chat-message-i__container`
- `chat-message-i__container--sent`
- `chat-message-i__container--received`
- `chat-message-i__message-container`
- `chat-message-i__avatar`
- `chat-message-i__header`
- `chat-message-i__header--public`
- `chat-message-i__header--private`
- `chat-message-i__message`

_Notes_:

Used in `CombinedChat` for a chat experience with a single thread, where differentiating between participants is more
difficult.

---

#### ChatMessageAttachment

Renders a chat attachment in a received chat message with a Download button, so the attachment can be downloaded.

_Props_ (`ChatMessageAttachmentElementProps`):

|        Property         |                          Type                          | Description                                                                                                                             |
|:-----------------------:|:------------------------------------------------------:|-----------------------------------------------------------------------------------------------------------------------------------------|
|      `attachment`       |      [ChatAttachmentModel](#chatattachmentmodel)       | The attachment to display.                                                                                                              |
| `getAttachmentLocation` | (fileId: string) => Promise\<string &#124; undefined>? | Function for getting a download URL for an attachment. Usually comes from `chatActions` provided by [useZocksActions](#usezocksactions) |

_Related CSS classes_:

- `chat-message-attachment__container`
- `chat-message-attachment__name`
- `chat-message-attachment__download-button`
- `chat-message-attachment__hidden-link`

_Notes_:

Uses the custom hook `useChatAttachment`. This hook provides download capabilities with loading indicator and error
handling for simple downloading of attachments via an
anchor tag.

Parameters:

|        Parameter        |                          Type                          | Description                                                                                                                              |
|:-----------------------:|:------------------------------------------------------:|------------------------------------------------------------------------------------------------------------------------------------------|
|      `attachment`       |      [ChatAttachmentModel](#chatattachmentmodel)       | The attachment to display.                                                                                                               |
| `getAttachmentLocation` | (fileId: string) => Promise\<string &#124; undefined>? | Function for getting a download URL for an attachment. Usually comes from `chatActions` provided by [useZocksActions](#usezocksactions). |

Returns:

|       Property        |                Type                 | Description                                                                    |
|:---------------------:|:-----------------------------------:|--------------------------------------------------------------------------------|
|      `anchorRef`      | React.RefObject\<HTMLAnchorElement> | `ref` to an invisible anchor tag which is used for downloading the attachment. |
| `downloadAttachment`  |             () => void              | Generates a new download URL and downloads the attachment via that URL         |
| `isAttachmentLoading` |               boolean               | Flag indicating attachment download URL generation is in progress              |
| `hasAttachmentError`  |               boolean               | Flag indicating if there was an error during download URL generation           |
|    `attachmentUrl`    |               string                | The generated download URL for the attachment for display purposes             |

### Full chat experience

#### CombinedChat

Provides a chat experience where there is a single thread and all messages, whether they are public or private, are
displayed in a single list.

_Props_ (`ChatProps`):

|       Property       |                 Type                  | Description                                                                                                     |
|:--------------------:|:-------------------------------------:|-----------------------------------------------------------------------------------------------------------------|
|    `chatActions`     | [ZocksChatActions](#zockschatactions) | Chat actions. Usually comes from `chatActions` provided by [useZocksActions](#usezocksactions).                 |
|      `dismiss`       |              () => void               | Optional. Called when dismissing the chat.                                                                      |
|  `chatInputElement`  |   React.FC\<ChatInputElementProps>    | Optional. Element renderer for customizing the chat input element. Default: `CustomizableChatInput`.            |
| `chatMessageElement` |  React.FC\<ChatMessageElementProps>   | Optional. Element renderer for customizing the chat message element. Default: `ChatMessageWithParticipantInfo`. |

_Related CSS classes_:

- `chat`
- `chat__header`
- `chat__main-container`
- `chat__messages-container`
- `chat__footer`

---

#### SplitChat

Provides a full chat experience with separate threads and a custom recipient selector.

_Props_ (`SplitChatProps`):

|          Property          |                   Type                   | Description                                                                                                     |
|:--------------------------:|:----------------------------------------:|-----------------------------------------------------------------------------------------------------------------|
|       `chatActions`        |  [ZocksChatActions](#zockschatactions)   | Chat actions. Usually comes from `chatActions` provided by [useZocksActions](#usezocksactions).                 |
|         `dismiss`          |                () => void                | Optional. Called when dismissing the chat.                                                                      |
|     `chatInputElement`     |     React.FC\<ChatInputElementProps>     | Optional. Element renderer for customizing the chat input element. Default: `CustomizableChatInput`.            |
|    `chatMessageElement`    |    React.FC\<ChatMessageElementProps>    | Optional. Element renderer for customizing the chat message element. Default: `ChatMessageWithParticipantInfo`. |
| `recipientSelectorElement` | React.FC\<RecipientSelectorElementProps> | Optional. Element renderer for customizing the recipient selector element. Default: `RecipientSelector`.        |

_Related CSS classes_:

- `chat`
- `chat__header`
- `chat__main-container`
- `chat__messages-container`
- `chat__footer`

