import React, { ChangeEvent, useRef, useState } from 'react'
import classNames from 'classnames'
import {
  AttachmentDisplayElementProps,
  AttachmentUploaderElementProps,
  ClearAttachmentButtonElementProps,
  UploadAttachmentButtonElementProps,
} from '../models'
import './Attachment.scss'

export const AttachmentUploader: React.FC<AttachmentUploaderElementProps> = ({
  uploadAttachment,
  attachmentRecipient,
  onAttachmentUploadStarted,
  onAttachmentUploadFinished,
  onAttachmentUploadFailed,
  uploadAttachmentButtonElement,
}) => {
  const fileInputRef = useRef<HTMLInputElement>(null)
  const [isUploading, setUploading] = useState(false)
  const [hasError, setHasError] = useState(false)

  const buttonClickHandler = () => {
    fileInputRef.current?.click()
  }

  const uploadStarted = (file: File) => {
    setHasError(false)
    setUploading(true)
    onAttachmentUploadStarted && onAttachmentUploadStarted(file)
  }

  const uploadFinished = (file: File, fileId?: string) => {
    if (fileId) {
      onAttachmentUploadFinished && onAttachmentUploadFinished(file, fileId)
    } else {
      uploadFailed(file, new Error('No file id returned.'))
    }
  }

  const uploadFailed = (file: File, error: Error) => {
    console.error(error)
    setHasError(true)
    onAttachmentUploadFailed && onAttachmentUploadFailed(file)
  }

  const fileUploadHandler = (e: ChangeEvent<HTMLInputElement>) => {
    const {
      target: { files },
    } = e
    if (files?.length) {
      const file = files[0]
      uploadStarted(file)
      uploadAttachment(file, attachmentRecipient)
        .then((fileId) => uploadFinished(file, fileId))
        .catch((error) => uploadFailed(file, error))
        .finally(() => setUploading(false))
      e.target.value = ''
    }
  }

  const UploadAttachmentButtonElement = uploadAttachmentButtonElement ?? UploadAttachmentButton

  return (
    <div className='attachment-uploader__container'>
      <UploadAttachmentButtonElement onClick={buttonClickHandler} disabled={isUploading} hasError={hasError} />
      <input
        ref={fileInputRef}
        className='attachment-uploader__file-input'
        type='file'
        onChange={fileUploadHandler}
        disabled={isUploading}
        aria-hidden={true}
        data-testid='hidden-file-input'
      />
    </div>
  )
}

const UploadAttachmentButton: React.FC<UploadAttachmentButtonElementProps> = ({ onClick, disabled, hasError }) => (
  <button
    className={classNames('attachment-uploader__open-button', {
      'attachment-uploader__open-button--error': hasError,
    })}
    aria-label='Upload a file'
    onClick={onClick}
    disabled={disabled}
  >
    📎
  </button>
)

export const AttachmentDisplay: React.FC<AttachmentDisplayElementProps> = ({
  attachment,
  onClearAttachment,
  hasAttachmentError,
  clearAttachmentButtonElement,
}) => {
  if (hasAttachmentError) {
    return <div className='attachment-display__error'>{'Error while uploading attachment.'}</div>
  }

  const ClearAttachmentButtonElement = clearAttachmentButtonElement ?? ClearAttachmentButton

  if (attachment) {
    return (
      <div className='attachment-display__container'>
        <span className='attachment-display__name' title={attachment.fileName}>
          {attachment.fileName}
        </span>
        {onClearAttachment && <ClearAttachmentButtonElement onClick={onClearAttachment} />}
      </div>
    )
  }

  return null
}

const ClearAttachmentButton: React.FC<ClearAttachmentButtonElementProps> = ({ onClick }) => (
  <button className='attachment-display__remove-button' onClick={onClick} aria-label='Remove attachment'>
    ❌
  </button>
)
