import React, { useEffect, useMemo, useRef } from 'react'
import { ChatMessageModel, useChat, selectRemoteParticipants } from '../../store'
import { useZocksStore } from '../../app'
import { ChatProps } from './models'
import { CustomizableChatInput } from './ChatInput'
import { ChatMessageWithParticipantInfo } from './ChatMessage'
import { useZocksDispatch } from '../../app/hooks'
import { setMessageRecipient } from '../../store/chat/chatSlice'
import './Chat.scss'

export const CombinedChat: React.FC<ChatProps> = ({ chatActions, dismiss, chatMessageElement, chatInputElement }) => {
  const dispatch = useZocksDispatch()
  const {
    sendPublicMessage,
    sendPrivateMessage,
    uploadAttachment,
    getAttachmentLocation,
    publicChatMessages,
    privateChatMessages,
    messageRecipient,
  } = useChat(chatActions)
  const remoteParticipants = useZocksStore(selectRemoteParticipants)

  const messagesEndRef = useRef<HTMLDivElement>(null)
  const scrollToBottom = () => {
    messagesEndRef.current?.scrollIntoView({ behavior: 'smooth' })
  }

  const allChatMessages = useMemo(
    () =>
      [...publicChatMessages, ...Object.values(privateChatMessages).flat()].sort((a, b) => a.timestamp - b.timestamp),
    [publicChatMessages, privateChatMessages]
  )

  useEffect(() => {
    scrollToBottom()
  }, [allChatMessages])

  const sameSenderAndRecipient = (message: ChatMessageModel, other: ChatMessageModel): boolean => {
    const {
      from: { id: messageSenderId },
      to: { id: messageRecipientId } = {},
    } = message
    const {
      from: { id: otherSenderId },
      to: { id: otherRecipientId } = {},
    } = other
    return messageSenderId === otherSenderId && messageRecipientId === otherRecipientId
  }

  const handleRecipientSelected = (recipientId: string) => {
    dispatch(setMessageRecipient(recipientId))
  }

  const ChatMessageElement = chatMessageElement ?? ChatMessageWithParticipantInfo
  const ChatInputElement = chatInputElement ?? CustomizableChatInput

  return (
    <div className='chat'>
      {dismiss && (
        <div className='chat__header'>
          <button className='chat__header-dismiss-button' onClick={dismiss} aria-label='Dismiss' />
        </div>
      )}
      <div className='chat__main-container'>
        <div className='chat__messages-container'>
          <div className='chat__messages-scroll'>
            {allChatMessages.map((chatMessage, idx) => {
              const showParticipantInfo =
                idx === 0 || !sameSenderAndRecipient(allChatMessages[idx], allChatMessages[idx - 1])
              return (
                <ChatMessageElement
                  key={idx}
                  chatMessage={chatMessage}
                  showParticipantInfo={showParticipantInfo}
                  getAttachmentLocation={getAttachmentLocation}
                />
              )
            })}
            <div ref={messagesEndRef} />
          </div>
        </div>
        <div className='chat__footer'>
          <ChatInputElement
            messageRecipient={messageRecipient}
            onRecipientSelected={handleRecipientSelected}
            remoteParticipants={remoteParticipants}
            sendPublicMessage={sendPublicMessage}
            sendPrivateMessage={sendPrivateMessage}
            uploadAttachment={uploadAttachment}
          />
        </div>
      </div>
    </div>
  )
}
