import React, { ChangeEvent, useState } from 'react'
import classNames from 'classnames'
import { ArrowContainer, Popover, PopoverPosition } from 'react-tiny-popover'
import { RoomInfo, VirtualBackgroundType } from '@zocks-communications/client-sdk-js'
import { useZocksStore } from '../../app'
import { JoinOptions, selectIsConnecting, useLocalMedia, useLocalVirtualBackgrounds } from '../../store'
import { Video } from '../Media'
import { DeviceSelectButton } from '../DeviceSelectButton'
import { AudioControlButtonProps, VideoControlButtonProps } from '../RoomControls'
import './JoinPage.scss'
import { VirtualBackgroundSelector, VirtualBackgroundSelectorProps } from '../VirtualBackgroundSelector'

export const JoinPage: React.FC<{
  joinAction: (joinOptions: JoinOptions) => void
  createRoomAction: () => Promise<RoomInfo>
  initialRoomId?: string
  joinText?: string
  newMeetingText?: string
  createNewMeetingText?: string
  newMeetingDisabled?: boolean
  roomIdInputLabel?: string
  roomIdInputErrorLabel?: string
  roomIdInputDisabled?: boolean
  roomIdPlaceholder?: string
  videoEnabledByDefault?: boolean
  audioEnabledByDefault?: boolean
  onNewRoomCreated?: (roomInfo: RoomInfo) => void
  dividerText?: string
  controlsHeaderElement?: string | JSX.Element
  disabledVideoElement?: string | JSX.Element
  audioControlButtonElement?: React.FC<AudioControlButtonProps>
  videoControlButtonElement?: React.FC<VideoControlButtonProps>
  virtualBackgroundSelectorElement?: React.FC<VirtualBackgroundSelectorProps>
  joinOptions?: Partial<JoinOptions>
}> = ({
  joinAction,
  createRoomAction,
  initialRoomId = '',
  joinText = 'Join',
  newMeetingText = 'New Meeting',
  createNewMeetingText = 'Create New Meeting',
  newMeetingDisabled = false,
  roomIdInputLabel = 'Room id',
  roomIdInputErrorLabel = 'Invalid room id!',
  roomIdInputDisabled = false,
  roomIdPlaceholder = 'Enter a room id',
  videoEnabledByDefault = false,
  audioEnabledByDefault = false,
  onNewRoomCreated = null,
  dividerText = null,
  controlsHeaderElement = null,
  disabledVideoElement = null,
  audioControlButtonElement = null,
  videoControlButtonElement = null,
  virtualBackgroundSelectorElement = null,
  joinOptions = {},
}) => {
  const isConnecting = useZocksStore(selectIsConnecting)
  const [roomId, setRoomId] = useState<string>(initialRoomId)
  const [hasError, setHasError] = useState<boolean>(false)
  const [isPopoverOpen, setPopoverOpen] = useState<boolean>(false)

  const {
    currentAudioDeviceId,
    currentVideoDeviceId,
    localAudioStream,
    localVideoStream,
    videoStreamIsLoading,
    toggleLocalAudioStream,
    toggleLocalVideoStream,
    selectAudioDevice,
    selectVideoDevice,
  } = useLocalMedia(videoEnabledByDefault, audioEnabledByDefault)

  const audioDeviceSettings = localAudioStream?.mediaStreamTrack?.getSettings()
  const videoDeviceSettings = localVideoStream?.mediaStreamTrack?.getSettings()

  const {
    availableVirtualBackgrounds,
    currentVirtualBackground,
    virtualBackgroundStream,
    virtualBackgroundIsLoading,
    changeVirtualBackground,
    stopLocalVirtualBackground,
  } = useLocalVirtualBackgrounds()

  const audioEnabled = !!localAudioStream
  const videoEnabled = !!localVideoStream || !!virtualBackgroundStream

  const afterJoinCallback = () => {
    localAudioStream && toggleLocalAudioStream()
    localVideoStream && toggleLocalVideoStream()
    currentVirtualBackground && stopLocalVirtualBackground()
  }

  const joinErrorCallback = () => {
    setPopoverOpen(true)
    setHasError(true)
  }

  const baseJoinOptions: JoinOptions = {
    roomId,
    autoCreate: false,
    audioDeviceSettings,
    videoDeviceSettings,
    virtualBackground: currentVirtualBackground,
    afterJoinCallback,
    joinErrorCallback,
    autoSubscribe: true,
    ...joinOptions,
  }

  const onRoomIdInputChange = (evt: ChangeEvent<HTMLInputElement>) => {
    setHasError(false)
    setPopoverOpen(false)
    setRoomId(evt.target.value)
  }

  const onRoomIdInputKeyUp = (evt: React.KeyboardEvent<HTMLInputElement>) => {
    if (roomId && evt.key === 'Enter') {
      joinAction(baseJoinOptions)
    }
  }

  const createNewMeeting = () => {
    createRoomAction().then((roomInfo: RoomInfo) => onNewRoomCreated && onNewRoomCreated(roomInfo))
  }

  const joinErrorTooltip = (position: PopoverPosition | undefined, childRect: ClientRect, popoverRect: ClientRect) => (
    <div className='join_page__room-id-input-popover'>
      <ArrowContainer
        position={position}
        childRect={childRect}
        popoverRect={popoverRect}
        arrowColor={'black'}
        arrowSize={10}
        className='popover-arrow-container'
        arrowClassName='popover-arrow'
      >
        <div className='join-page__room-id-input-popover-text'>{roomIdInputErrorLabel}</div>
      </ArrowContainer>
    </div>
  )

  const changeVideoDevice = (videoDevice: MediaDeviceInfo) => {
    selectVideoDevice(videoDevice)
    if (currentVirtualBackground) {
      changeVirtualBackground(currentVirtualBackground, videoDevice.deviceId)
    }
  }

  const selectVirtualBackground = (virtualBackground: VirtualBackgroundType | null) => {
    changeVirtualBackground(virtualBackground, currentVideoDeviceId || undefined)
  }

  const toggleVideoStream = () => {
    toggleLocalVideoStream()
    if (videoEnabled) {
      stopLocalVirtualBackground()
    } else if (currentVirtualBackground) {
      changeVirtualBackground(currentVirtualBackground, currentVideoDeviceId || undefined)
    }
  }

  const AudioControlButtonElement = audioControlButtonElement ?? AudioControlButton
  const VideoControlButtonElement = videoControlButtonElement ?? VideoControlButton
  const VirtualBackgroundSelectorElement = virtualBackgroundSelectorElement ?? VirtualBackgroundSelector

  return (
    <div className='join-page'>
      <div className='join-page__local-streams'>
        {localVideoStream || virtualBackgroundStream ? (
          <div className='join-page__video-container'>
            <div className='join-page__video-preview'>
              <Video videoStream={virtualBackgroundStream || localVideoStream!} />
            </div>
          </div>
        ) : (
          <div className='join-page__no-video-icon'>{disabledVideoElement}</div>
        )}
        <div className='join-page__local-stream-buttons'>
          <AudioControlButtonElement
            audioIsOn={audioEnabled}
            currentDeviceId={currentAudioDeviceId}
            toggleAudio={toggleLocalAudioStream}
            onDeviceSelected={selectAudioDevice}
          />
          <VideoControlButtonElement
            videoIsOn={videoEnabled}
            toggleVideo={toggleVideoStream}
            onDeviceSelected={changeVideoDevice}
            videoIsLoading={videoStreamIsLoading}
            currentDeviceId={currentVideoDeviceId}
            availableVirtualBackgrounds={availableVirtualBackgrounds}
            selectVirtualBackground={selectVirtualBackground}
            currentVirtualBackground={currentVirtualBackground}
            virtualBackgroundIsLoading={virtualBackgroundIsLoading}
          />
          <VirtualBackgroundSelectorElement
            currentVirtualBackground={currentVirtualBackground}
            availableVirtualBackgrounds={availableVirtualBackgrounds}
            selectVirtualBackground={selectVirtualBackground}
            virtualBackgroundIsLoading={virtualBackgroundIsLoading}
          />
        </div>
      </div>
      <div className='join-page__controls'>
        <div className='join-page__join-room-container'>
          {controlsHeaderElement}
          <Popover
            isOpen={isPopoverOpen}
            content={({ position, childRect, popoverRect }) => joinErrorTooltip(position, childRect, popoverRect)}
            positions={['top']}
          >
            <input
              onMouseEnter={() => setPopoverOpen(hasError)}
              onMouseLeave={() => setPopoverOpen(false)}
              className={classNames({ 'join-page__room-id-input--error': hasError })}
              type='text'
              onChange={onRoomIdInputChange}
              onKeyUp={onRoomIdInputKeyUp}
              value={roomId}
              placeholder={roomIdPlaceholder}
              aria-label={roomIdInputLabel}
              disabled={roomIdInputDisabled}
            />
          </Popover>
          <button
            className='join-page__button join-page__join-button'
            onClick={() => joinAction(baseJoinOptions)}
            disabled={!roomId || isConnecting || hasError}
          >
            {joinText}
          </button>
        </div>
        <div className='join-page__controls-divider'>{dividerText ? <span>{dividerText}</span> : ''}</div>
        <div className='join-page__new-meeting-container'>
          <button
            className='join-page__button join-page__button join-page__new-meeting-button'
            onClick={() => joinAction({ ...baseJoinOptions, roomId: '', autoCreate: true })}
            disabled={isConnecting || newMeetingDisabled}
          >
            {newMeetingText}
          </button>
          <button
            className='join-page__button join-page__button join-page__create-new-meeting-button'
            onClick={createNewMeeting}
            disabled={isConnecting || newMeetingDisabled}
          >
            {createNewMeetingText}
          </button>
        </div>
      </div>
    </div>
  )
}

const AudioControlButton: React.FC<AudioControlButtonProps> = ({ audioIsOn, toggleAudio, onDeviceSelected }) => {
  return (
    <div
      className={classNames('join-page__audio-button-container', {
        disabled: !audioIsOn,
      })}
    >
      <button
        id='join-page__audio-button'
        className={classNames('join-page__button join-page__audio-button', {
          'join-page__audio-button--enabled': audioIsOn,
          'join-page__audio-button--disabled': !audioIsOn,
        })}
        onClick={toggleAudio}
      >
        <label htmlFor='join-page__audio-button'>{audioIsOn ? 'Mute' : 'Unmute'}</label>
      </button>
      {onDeviceSelected && (
        <DeviceSelectButton
          className='join-page--audio-select-button'
          deviceKind='audioinput'
          onDeviceSelected={onDeviceSelected}
          label='Audio devices'
        />
      )}
    </div>
  )
}

const VideoControlButton: React.FC<VideoControlButtonProps> = ({
  videoIsOn,
  toggleVideo,
  onDeviceSelected,
  videoIsLoading,
}) => {
  return (
    <div
      className={classNames('join-page__video-button-container', {
        disabled: !videoIsOn,
      })}
    >
      <button
        id='join-page__video-button'
        disabled={videoIsLoading}
        className={classNames('join-page__button join-page__video-button', {
          'join-page__video-button--enabled': videoIsOn,
          'join-page__video-button--disabled': !videoIsOn,
        })}
        onClick={toggleVideo}
      >
        <label htmlFor='join-page__video-button'>{videoIsOn ? 'Disable Video' : 'Enable Video'}</label>
      </button>
      {onDeviceSelected && (
        <DeviceSelectButton
          disabled={videoIsLoading}
          className='join-page--video-select-button'
          deviceKind='videoinput'
          onDeviceSelected={onDeviceSelected}
          label='Video devices'
        />
      )}
    </div>
  )
}
