import React, { useEffect, useState } from 'react'
import classNames from 'classnames'
import { useParticipant } from '../../store'
import { Audio, Video } from '../Media'
import { ParticipantProps } from './model'
import './Participant.scss'

const SPEAKING_INDICATOR_TURN_OFF_DELAY = 1500

export const Participant: React.FC<ParticipantProps> = ({ participant, withAudio = false, extensionElement }) => {
  const {
    id,
    name,
    tracks: { primaryVideoTrack, primaryAudioTrack },
    isLocal,
    connectionQuality,
    isSpeaking,
    isScreenSharing,
  } = useParticipant(participant)

  const [showSpeaking, setShowSpeaking] = useState(isSpeaking)

  useEffect(() => {
    if (!isSpeaking) {
      const timeout = setTimeout(() => {
        setShowSpeaking(isSpeaking)
      }, SPEAKING_INDICATOR_TURN_OFF_DELAY)
      return () => clearTimeout(timeout)
    } else {
      setShowSpeaking(true)
    }
  }, [isSpeaking])

  const videoIsMuted = !primaryVideoTrack || !primaryVideoTrack.isSubscribed || primaryVideoTrack.isMuted
  const audioIsMuted = !primaryAudioTrack || !primaryAudioTrack.isSubscribed || primaryAudioTrack.isMuted

  const ExtensionElement = extensionElement || null

  return (
    <div
      id={`participant_${id}`}
      className={classNames('participant', {
        'participant__video--muted': videoIsMuted,
        'participant__audio--muted': audioIsMuted,
      })}
    >
      <div className='participant__name'>{name}</div>
      <div
        className={classNames('participant__video-container', {
          'participant__video-container--speaking': showSpeaking,
          'participant__video-container--local': isLocal,
        })}
      >
        {videoIsMuted ? (
          <div className='participant__avatar'>{name?.slice(0, 1) || ''}</div>
        ) : (
          <Video videoStream={primaryVideoTrack.track} />
        )}
      </div>
      <div className='participant__status-icons'>
        {videoIsMuted && <div className='participant__no-video-icon' />}
        {audioIsMuted && <div className='participant__no-audio-icon' />}
        <div className={`participant__${connectionQuality}-quality-icon`} />
      </div>
      {ExtensionElement && (
        <ExtensionElement
          participantId={id}
          participantName={name}
          isLocal={isLocal}
          isSpeaking={showSpeaking}
          connectionQuality={connectionQuality}
          videoIsMuted={videoIsMuted}
          audioIsMuted={audioIsMuted}
          isScreenSharing={isScreenSharing}
        />
      )}
      {withAudio && !audioIsMuted && !isLocal && <Audio audioStream={primaryAudioTrack.track} />}
    </div>
  )
}
