import React from 'react'
import classNames from 'classnames'
import { useRoomControls, useVirtualBackgrounds } from './hooks'
import { DeviceSelectButton } from '../DeviceSelectButton'
import { VirtualBackgroundSelector } from '../VirtualBackgroundSelector'
import {
  AudioControlButtonProps,
  LeaveButtonProps,
  RoomControlsProps,
  ScreenShareControlButtonProps,
  VideoControlButtonProps,
} from './models'
import './RoomControls.scss'

export const RoomControls: React.FC<RoomControlsProps> = ({
  mediaActions,
  leaveAction,
  audioControlButtonElement,
  videoControlButtonElement,
  screenShareControlButtonElement,
  leaveButtonElement,
  virtualBackgroundSelectorElement,
}) => {
  const {
    audioIsOn,
    videoIsOn,
    isScreenSharing,
    isLocalScreenShare,
    currentAudioDeviceId,
    currentVideoDeviceId,
    toggleAudio,
    toggleVideo,
    toggleScreenShare,
    selectAudioDevice,
    selectVideoDevice,
  } = useRoomControls(mediaActions)

  const { availableBackgrounds, virtualBackgroundIsLoading, currentVirtualBackground, changeVirtualBackground } =
    useVirtualBackgrounds(mediaActions)

  const AudioControlButtonElement = audioControlButtonElement ?? AudioControlButton
  const VideoControlButtonElement = videoControlButtonElement ?? VideoControlButton
  const ScreenShareControlButtonElement = screenShareControlButtonElement ?? ScreenShareControlButton
  const LeaveButtonElement = leaveButtonElement ?? LeaveButton
  const VirtualBackgroundSelectorElement = virtualBackgroundSelectorElement ?? VirtualBackgroundSelector

  return (
    <>
      <AudioControlButtonElement
        audioIsOn={audioIsOn}
        toggleAudio={toggleAudio}
        currentDeviceId={currentAudioDeviceId}
        onDeviceSelected={selectAudioDevice}
      />
      <VideoControlButtonElement
        videoIsOn={videoIsOn}
        toggleVideo={toggleVideo}
        currentDeviceId={currentVideoDeviceId}
        onDeviceSelected={selectVideoDevice}
        availableVirtualBackgrounds={availableBackgrounds}
        selectVirtualBackground={changeVirtualBackground}
        currentVirtualBackground={currentVirtualBackground}
        virtualBackgroundIsLoading={virtualBackgroundIsLoading}
      />
      <ScreenShareControlButtonElement
        isScreenSharing={isScreenSharing}
        isLocalScreenShare={isLocalScreenShare}
        toggleScreenShare={toggleScreenShare}
      />
      <VirtualBackgroundSelectorElement
        currentVirtualBackground={currentVirtualBackground}
        availableVirtualBackgrounds={availableBackgrounds}
        selectVirtualBackground={changeVirtualBackground}
        virtualBackgroundIsLoading={virtualBackgroundIsLoading}
      />
      <LeaveButtonElement leaveAction={leaveAction} />
    </>
  )
}

const AudioControlButton: React.FC<AudioControlButtonProps> = ({ audioIsOn, toggleAudio, onDeviceSelected }) => {
  return (
    <div className='room-controls__audio-button-container'>
      <button id='room-controls__audio-button' className='room-controls__control-button' onClick={toggleAudio}>
        <div
          className={classNames({
            'room-controls__audio-button--unmuted': audioIsOn,
            'room-controls__audio-button--muted': !audioIsOn,
          })}
        />
        <label htmlFor='room-controls__audio-button'>{audioIsOn ? 'Mute' : 'Unmute'}</label>
      </button>
      {onDeviceSelected && (
        <DeviceSelectButton
          className='room-controls--audio-select-button'
          deviceKind={'audioinput'}
          onDeviceSelected={onDeviceSelected}
          label={'Audio devices'}
        />
      )}
    </div>
  )
}

const VideoControlButton: React.FC<VideoControlButtonProps> = ({ videoIsOn, toggleVideo, onDeviceSelected }) => {
  return (
    <div className='room-controls__video-button-container'>
      <button id='room-controls__video-button' className='room-controls__control-button' onClick={toggleVideo}>
        <div
          className={classNames({
            'room-controls__video-button--unmuted': videoIsOn,
            'room-controls__video-button--muted': !videoIsOn,
          })}
        />
        <label htmlFor='room-controls__video-button'>{videoIsOn ? 'Disable Video' : 'Enable Video'}</label>
      </button>
      {onDeviceSelected && (
        <DeviceSelectButton
          className='room-controls--video-select-button'
          deviceKind={'videoinput'}
          onDeviceSelected={onDeviceSelected}
          label={'Video devices'}
        />
      )}
    </div>
  )
}

const ScreenShareControlButton: React.FC<ScreenShareControlButtonProps> = ({
  isScreenSharing,
  isLocalScreenShare,
  toggleScreenShare,
}) => {
  return (
    <button
      disabled={isScreenSharing && !isLocalScreenShare}
      id='room-controls__screen-share-button'
      className='room-controls__control-button room-controls__screen-share-button'
      onClick={toggleScreenShare}
    >
      <div
        className={classNames({
          'room-controls__screen-share-button--share-on': !isScreenSharing || !isLocalScreenShare,
          'room-controls__screen-share-button--share-off': isScreenSharing && isLocalScreenShare,
        })}
      />
      <label htmlFor='room-controls__screen-share-button'>{isLocalScreenShare ? 'Stop Sharing' : 'Share Screen'}</label>
    </button>
  )
}

const LeaveButton: React.FC<LeaveButtonProps> = ({ leaveAction }) => {
  return (
    <button className='room-controls__leave-button room-controls__control-button' onClick={leaveAction}>
      <label htmlFor='room-controls__leave-button'>Leave</label>
    </button>
  )
}
