import './OneOnOneParticipantView.scss'
import '../../../assets/css/zocksicons.css'
import React from 'react'

import classNames from 'classnames'
import { VirtualBackgroundType } from '@zocks-communications/client-sdk-js'
import { FullScreenButtonProps, JoinButtonProps, OneOnOneParticipantProps } from './models'
import { Loader } from './loader/Loader'
import {
  ActiveSpeakerParticipant,
  AudioControlButtonProps,
  DeviceSelectButton,
  Participant,
  RoomControls,
  ScreenShare,
  useSortRemoteParticipantsBySpeakers,
  Video,
  VideoControlButtonProps,
} from '../../../components'
import {
  JoinOptions,
  selectIsConnecting,
  selectScreenSharingParticipant,
  useLocalMedia,
  useLocalVirtualBackgrounds,
  useZocksSession,
} from '../../../store'
import { useZocksStore } from '../../../app'
import { createVideoControlMenu } from './CustomVideoControl'
import { useFullScreen } from './hooks'
import { createAudioControlMenu } from './CustomAudioControl'

export const OneOnOneParticipantView: React.FC<OneOnOneParticipantProps> = ({
  zocksActions,
  roomId,
  emptyJoinPageText = 'Your camera is turned off.',
  participantNameHidden = false,
  emptyRemoteParticipantText = 'Waiting for other participant to join.',
  joinButtonElement,
  joinLoaderElement,
  localParticipantElement,
  remoteParticipantElement,
  audioControlButtonElement,
  videoControlButtonElement,
  screenShareControlButtonElement,
  leaveButtonElement,
  fullScreenButtonElement,
}) => {
  const screenSharingParticipant = useZocksStore(selectScreenSharingParticipant)
  const { isInFullscreen, toggleFullScreen } = useFullScreen()
  const { remoteParticipantsSortedBySpeakers } = useSortRemoteParticipantsBySpeakers()
  const { localParticipantIsConnected, localParticipant } = useZocksSession()
  const remoteParticipantsExist = !!remoteParticipantsSortedBySpeakers.length

  const mainParticipant = remoteParticipantsExist ? remoteParticipantsSortedBySpeakers[0] : localParticipant

  const FloatingParticipantElement = localParticipantElement ?? Participant
  const RemoteParticipantElement = remoteParticipantElement ?? Participant

  const AudioControlButtonElement = audioControlButtonElement ?? CustomAudioControlButton
  const VideoControlButtonElement = videoControlButtonElement ?? CustomVideoControlButton
  const FullScreenButtonElement = fullScreenButtonElement ?? FullScreenButton

  return (
    <div className='oneonone-participants-view'>
      {!localParticipantIsConnected ? (
        <JoinPageInVideoView
          sessionId={roomId}
          emptyJoinPageText={emptyJoinPageText}
          joinLoaderElement={joinLoaderElement}
          joinAction={zocksActions.sessionActions.join}
          joinButtonElement={joinButtonElement}
        />
      ) : (
        <div
          className={classNames('oneonone-participants-container', {
            'oneonone-participants-view--full-screen': isInFullscreen,
          })}
        >
          <div
            className={classNames('oneonone-participants-view__active-speaker', {
              'oneonone-participants-view__only-speaker': !remoteParticipantsExist,
            })}
          >
            {screenSharingParticipant ? (
              <div className='oneonone-participants-screen-share'>
                <ScreenShare participant={screenSharingParticipant} />
              </div>
            ) : (
              <div
                className={classNames('oneonone-participant-container', {
                  'oneonone-participant-container__participant--name-hidden': participantNameHidden,
                })}
              >
                {remoteParticipantsExist ? (
                  <ActiveSpeakerParticipant
                    mainParticipant={mainParticipant}
                    participantElement={RemoteParticipantElement}
                  />
                ) : (
                  <div className='oneoneone-participant-container__remote-participant--not-connected'>
                    {emptyRemoteParticipantText}
                  </div>
                )}
              </div>
            )}
          </div>
          <div className='oneonone-participants-view__local-participant'>
            <div
              className='oneonone-participant-container
               oneonone-participant-container__participant--name-hidden
               oneonone-participant-container__hide_participant-status-icons'
            >
              <FloatingParticipantElement participant={screenSharingParticipant ? mainParticipant : localParticipant} />
            </div>
          </div>

          <div className='oneonone-participants-view__room-controls oneonone-room-controls'>
            <RoomControls
              mediaActions={zocksActions.localMediaActions}
              leaveAction={zocksActions.sessionActions.leave}
              audioControlButtonElement={AudioControlButtonElement}
              videoControlButtonElement={VideoControlButtonElement}
              screenShareControlButtonElement={screenShareControlButtonElement}
              leaveButtonElement={leaveButtonElement}
              virtualBackgroundSelectorElement={() => null}
            />
            <FullScreenButtonElement isInFullscreen={isInFullscreen} toggleFullscreen={toggleFullScreen} />
          </div>
        </div>
      )}
    </div>
  )
}

const JoinPageInVideoView: React.FC<{
  sessionId: string
  joinAction: (joinOptions: JoinOptions) => void
  joinOptions?: Partial<JoinOptions>
  emptyJoinPageText?: string
  joinButtonElement?: React.FC<JoinButtonProps>
  joinLoaderElement?: React.FC
}> = ({ sessionId, joinAction, joinOptions = {}, emptyJoinPageText, joinButtonElement, joinLoaderElement }) => {
  const {
    currentAudioDeviceId,
    currentVideoDeviceId,
    localAudioStream,
    localVideoStream,
    videoStreamIsLoading,
    toggleLocalAudioStream,
    toggleLocalVideoStream,
    selectAudioDevice,
    selectVideoDevice,
  } = useLocalMedia(false, false)

  const {
    availableVirtualBackgrounds,
    currentVirtualBackground,
    virtualBackgroundStream,
    virtualBackgroundIsLoading,
    changeVirtualBackground,
    stopLocalVirtualBackground,
  } = useLocalVirtualBackgrounds()
  const audioDeviceSettings = localAudioStream?.mediaStreamTrack?.getSettings()
  const videoDeviceSettings = localVideoStream?.mediaStreamTrack?.getSettings()

  const afterJoinCallback = () => {
    localAudioStream && toggleLocalAudioStream()
    localVideoStream && toggleLocalVideoStream()
    currentVirtualBackground && stopLocalVirtualBackground()
  }
  const isConnecting = useZocksStore(selectIsConnecting)

  const joinErrorCallback = () => {}

  const baseJoinOptions: JoinOptions = {
    roomId: sessionId,
    autoCreate: false,
    audioDeviceSettings,
    videoDeviceSettings,
    virtualBackground: currentVirtualBackground,
    afterJoinCallback,
    joinErrorCallback,
    autoSubscribe: true,
    ...joinOptions,
  }

  const changeVideoDevice = (videoDevice: MediaDeviceInfo) => {
    selectVideoDevice(videoDevice)
    if (currentVirtualBackground) {
      changeVirtualBackground(currentVirtualBackground, videoDevice.deviceId)
    }
  }

  const selectVirtualBackground = (virtualBackground: VirtualBackgroundType | null) => {
    changeVirtualBackground(virtualBackground, currentVideoDeviceId || undefined)
  }

  const toggleVideoStream = () => {
    toggleLocalVideoStream()
    if (videoEnabled) {
      stopLocalVirtualBackground()
    } else if (currentVirtualBackground) {
      changeVirtualBackground(currentVirtualBackground, currentVideoDeviceId || undefined)
    }
  }

  const JoinButtonElement = joinButtonElement ? joinButtonElement : JoinButton
  const JoinLoaderElement = joinLoaderElement || Loader
  const audioEnabled = !!localAudioStream
  const videoEnabled = !!localVideoStream || !!virtualBackgroundStream

  return (
    <div className='join-page-in-video__local-streams'>
      {videoEnabled ? (
        <div className='join-page-in-video__video-container'>
          <div className='join-page-in-video__video-preview'>
            {isConnecting ? (
              <JoinLoaderElement />
            ) : (
              <Video videoStream={virtualBackgroundStream || localVideoStream!} />
            )}
          </div>
        </div>
      ) : (
        <div className='join-page-in-video__no-video-icon'>
          {isConnecting ? joinLoaderElement ? <JoinLoaderElement /> : <Loader /> : emptyJoinPageText}
        </div>
      )}
      <div className='join-page-in-video__local-stream-buttons'>
        <CustomAudioControlButton
          audioIsOn={audioEnabled}
          currentDeviceId={currentAudioDeviceId}
          toggleAudio={toggleLocalAudioStream}
          onDeviceSelected={selectAudioDevice}
        />
        <CustomVideoControlButton
          videoIsOn={videoEnabled}
          toggleVideo={toggleVideoStream}
          onDeviceSelected={changeVideoDevice}
          videoIsLoading={videoStreamIsLoading}
          currentDeviceId={currentVideoDeviceId}
          availableVirtualBackgrounds={availableVirtualBackgrounds}
          selectVirtualBackground={selectVirtualBackground}
          currentVirtualBackground={currentVirtualBackground}
          virtualBackgroundIsLoading={virtualBackgroundIsLoading}
        />
        <div>
          <JoinButtonElement joinAction={joinAction} joinOptions={baseJoinOptions} isConnecting={isConnecting} />
        </div>
      </div>
    </div>
  )
}

const JoinButton: React.FC<JoinButtonProps> = ({ joinAction, joinOptions, isConnecting }) => {
  return (
    <button
      className='join-page-in-video__button join-page-in-video__join-button'
      onClick={() => joinAction(joinOptions)}
      disabled={isConnecting}
    >
      <label htmlFor='join-page-in-video__join-button'>Join</label>
    </button>
  )
}

const FullScreenButton: React.FC<FullScreenButtonProps> = ({ isInFullscreen, toggleFullscreen }) => {
  return (
    <button
      className='oneonone-room-controls__control-button oneonone-room-controls__full-screen-button'
      onClick={toggleFullscreen}
    >
      <div
        className={classNames({
          'oneonone-room-controls__full-screen-button--on': !isInFullscreen,
          'oneonone-room-controls__full-screen-button--off': isInFullscreen,
        })}
      />
      <label htmlFor='oneonone-room-controls__full-screen-button'>
        {isInFullscreen ? 'Exit Full Screen' : 'Full Screen'}
      </label>
    </button>
  )
}

const CustomVideoControlButton: React.FC<VideoControlButtonProps> = ({
  onDeviceSelected,
  toggleVideo,
  videoIsOn,
  currentVirtualBackground,
  selectVirtualBackground,
  virtualBackgroundIsLoading,
}) => {
  const VideoControlMenu = createVideoControlMenu(currentVirtualBackground, selectVirtualBackground)

  return (
    <div className='oneonone-room-controls__video-button-container'>
      <button className='oneonone-room-controls__control-button' onClick={toggleVideo}>
        <div
          className={classNames({
            'oneonone-room-controls__video-button--unmuted': videoIsOn,
            'oneonone-room-controls__video-button--muted': !videoIsOn,
          })}
        />
        <label htmlFor='oneonone-room-controls__video-button'>{videoIsOn ? 'Disable Video' : 'Enable Video'}</label>
      </button>
      {onDeviceSelected && (
        <DeviceSelectButton
          disabled={virtualBackgroundIsLoading || !videoIsOn}
          className='oneonone-room-controls__video-select-button'
          deviceKind={'videoinput'}
          onDeviceSelected={onDeviceSelected}
          label={'Video devices'}
          selectMenuElement={VideoControlMenu}
        />
      )}
    </div>
  )
}

const CustomAudioControlButton: React.FC<AudioControlButtonProps> = ({ onDeviceSelected, toggleAudio, audioIsOn }) => {
  const AudioControlMenu = createAudioControlMenu()

  return (
    <div className={classNames('oneonone-room-controls__audio-button-container')}>
      <button className='oneonone-room-controls__control-button' onClick={toggleAudio}>
        <div
          className={classNames({
            'oneonone-room-controls__audio-button--unmuted': audioIsOn,
            'oneonone-room-controls__audio-button--muted': !audioIsOn,
          })}
        />
        <label htmlFor='oneonone-room-controls__audio-button'>{audioIsOn ? 'Mute' : 'Unmute'}</label>
      </button>
      {onDeviceSelected && (
        <DeviceSelectButton
          disabled={!audioIsOn}
          className='oneonone-room-controls__audio-select-button'
          deviceKind={'audioinput'}
          onDeviceSelected={onDeviceSelected}
          label={'Audio devices'}
          selectMenuElement={AudioControlMenu}
        />
      )}
    </div>
  )
}
